@@ -1,23 +1,36 @@
 from keras_cv_attention_models import backend
 
-from .version import __version__
+from keras_cv_attention_models.version import __version__
+from keras_cv_attention_models import plot_func
 from keras_cv_attention_models import attention_layers
 from keras_cv_attention_models import beit
 from keras_cv_attention_models.beit import flexivit
 from keras_cv_attention_models.beit import eva
+from keras_cv_attention_models.beit import eva02
+from keras_cv_attention_models.beit import dinov2
+from keras_cv_attention_models.beit import meta_transformer
+from keras_cv_attention_models.beit import vit
 from keras_cv_attention_models import botnet
 from keras_cv_attention_models import caformer
+from keras_cv_attention_models import coat
 from keras_cv_attention_models import coatnet
 from keras_cv_attention_models import convnext
 from keras_cv_attention_models import cotnet
 from keras_cv_attention_models import cmt
+from keras_cv_attention_models import cspnext
 from keras_cv_attention_models import davit
 from keras_cv_attention_models import efficientnet
 from keras_cv_attention_models import edgenext
 from keras_cv_attention_models import efficientformer
+from keras_cv_attention_models import fasternet
 from keras_cv_attention_models import gcvit
 from keras_cv_attention_models import ghostnet
-from keras_cv_attention_models import ghostnet as ghostnetv2  # Will be removed
+from keras_cv_attention_models import ghostnet as ghostnetv2  # alias name
+from keras_cv_attention_models import gpt2
+from keras_cv_attention_models import llama2
+from keras_cv_attention_models import halonet
+from keras_cv_attention_models import hiera
+from keras_cv_attention_models import hornet
 from keras_cv_attention_models import iformer
 from keras_cv_attention_models import levit
 from keras_cv_attention_models import mlp_family
@@ -29,10 +42,17 @@ from keras_cv_attention_models.mobilenetv3_family import fbnetv3
 from keras_cv_attention_models.mobilenetv3_family import lcnet
 from keras_cv_attention_models.mobilenetv3_family import mobilenetv3
 from keras_cv_attention_models.mobilenetv3_family import tinynet
+from keras_cv_attention_models import efficientvit
+from keras_cv_attention_models.efficientvit import efficientvit_b
+from keras_cv_attention_models.efficientvit import efficientvit_m
+from keras_cv_attention_models import inceptionnext
 from keras_cv_attention_models import maxvit
 from keras_cv_attention_models import mobilevit
 from keras_cv_attention_models import moganet
+from keras_cv_attention_models import nat
+from keras_cv_attention_models.nat import dinat
 from keras_cv_attention_models import pvt
+from keras_cv_attention_models import repvit
 from keras_cv_attention_models import tinyvit
 from keras_cv_attention_models import resnest
 from keras_cv_attention_models import resnet_family
@@ -43,6 +63,9 @@ from keras_cv_attention_models.resnet_family import regnet
 from keras_cv_attention_models import gpvit
 from keras_cv_attention_models import swin_transformer_v2
 from keras_cv_attention_models import uniformer
+from keras_cv_attention_models import fastervit
+from keras_cv_attention_models import fastvit
+from keras_cv_attention_models import vanillanet
 from keras_cv_attention_models import download_and_load
 from keras_cv_attention_models import imagenet
 from keras_cv_attention_models import test_images
@@ -51,13 +74,16 @@ from keras_cv_attention_models import efficientdet
 from keras_cv_attention_models import yolox
 from keras_cv_attention_models import yolor
 from keras_cv_attention_models import yolov7
+from keras_cv_attention_models import yolov8
+from keras_cv_attention_models.yolov8 import yolo_nas
 from keras_cv_attention_models import coco
+from keras_cv_attention_models import clip
+from keras_cv_attention_models.clip import tokenizer
+from keras_cv_attention_models import stable_diffusion
+from keras_cv_attention_models import segment_anything
+from keras_cv_attention_models import segment_anything as sam  # Alias name
 
 if backend.is_tensorflow_backend:
-    from keras_cv_attention_models import coat
-    from keras_cv_attention_models import halonet
-    from keras_cv_attention_models import hornet
-    from keras_cv_attention_models import nat
     from keras_cv_attention_models import nfnets
     from keras_cv_attention_models import volo
     from keras_cv_attention_models import visualizing
