@@ -1,5 +1,6 @@
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models, functional, image_data_format
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import (
     activation_by_name,
     anti_alias_downsample,
@@ -72,11 +73,11 @@ def attn_block(
         nn = attention_layers.outlook_attention(nn, filters, **attn_params, name=name + "outlook_")
         need_downsaple = True
     # elif attn_type == "groups_conv":  # ResNeXt like
-    #     nn = conv2d_no_bias(nn, filters, **attn_params, strides=strides, padding="SAME", name=name + "GC_")
+    #     nn = conv2d_no_bias(nn, filters, **attn_params, strides=strides, padding="same", name=name + "GC_")
     else:  # ResNet and `groups > 1` for ResNeXt like
         groups = groups if group_size == 0 else filters // group_size
         conv_name = (name + "GC_") if groups > 1 else name
-        nn = conv2d_no_bias(nn, filters, 3, strides=strides, padding="SAME", groups=groups, name=conv_name)
+        nn = conv2d_no_bias(nn, filters, 3, strides=strides, padding="same", groups=groups, name=conv_name)
 
     # if strides != 1 and (nn.shape[1] == inputs.shape[1] or nn.shape[2] == inputs.shape[2]):  # Downsample
     if strides != 1 and need_downsaple:  # Downsample
@@ -100,7 +101,7 @@ def conv_shortcut_branch(inputs, filters, preact=False, strides=1, shortcut_type
         return None
 
     if strides > 1 and shortcut_type == "avg":
-        shortcut = layers.AvgPool2D(strides, strides=strides, padding="SAME", name=name + "shortcut_down")(inputs)
+        shortcut = layers.AvgPool2D(strides, strides=strides, padding="same", name=name + "shortcut_down")(inputs)
         strides = 1
     elif strides > 1 and shortcut_type == "anti_alias":
         shortcut = anti_alias_downsample(inputs, kernel_size=3, strides=2, name=name + "shortcut_down")
@@ -118,17 +119,17 @@ def deep_branch(
 ):
     hidden_filter = int(filters * hidden_channel_ratio)
     if use_3x3_kernel:
-        nn = conv2d_no_bias(inputs, hidden_filter, 3, strides=1, padding="SAME", name=name + "deep_1_")  # Using strides=1 for not changing input shape
-        # nn = conv2d_no_bias(inputs, hidden_filter, 3, strides=strides, padding="SAME", name=name + "1_")
+        nn = conv2d_no_bias(inputs, hidden_filter, 3, strides=1, padding="same", name=name + "deep_1_")  # Using strides=1 for not changing input shape
+        # nn = conv2d_no_bias(inputs, hidden_filter, 3, strides=strides, padding="same", name=name + "1_")
         # strides = 1
     else:
-        nn = conv2d_no_bias(inputs, hidden_filter, 1, strides=1, padding="VALID", name=name + "deep_1_")
+        nn = conv2d_no_bias(inputs, hidden_filter, 1, strides=1, padding="valid", name=name + "deep_1_")
     nn = batchnorm_with_activation(nn, zero_gamma=False, **bn_act_params, name=name + "deep_1_")
     # bn_after_attn = False if use_3x3_kernel else bn_after_attn
     nn = attn_block(nn, hidden_filter, strides, **attn_block_params, **bn_act_params, bn_after_attn=bn_after_attn, name=name + "deep_2_")
 
     if not use_3x3_kernel:
-        nn = conv2d_no_bias(nn, filters, 1, strides=1, padding="VALID", name=name + "deep_3_")
+        nn = conv2d_no_bias(nn, filters, 1, strides=1, padding="valid", name=name + "deep_3_")
     return nn
 
 
@@ -173,7 +174,7 @@ def aot_block(
         # short_act = activation if attn_block_params["attn_type"] == "bot" else None
         shortcut = conv_shortcut_branch(pre_inputs, filters, preact, strides, shortcut_type, bn_params, name=name)  # activation=None
     else:
-        shortcut = layers.MaxPool2D(strides, strides=strides, padding="SAME")(inputs) if strides > 1 else inputs
+        shortcut = layers.MaxPool2D(strides, strides=strides, padding="same")(inputs) if strides > 1 else inputs
 
     deep = deep_branch(pre_inputs, filters, strides, hidden_channel_ratio, use_3x3_kernel, bn_after_attn, bn_act_params, attn_block_params, name=name)
 
@@ -319,7 +320,7 @@ def AotNet(
 ):
     """Stem"""
     # Regard input_shape as force using original shape if len(input_shape) == 4,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
     inputs = layers.Input(shape=input_shape)
     bn_params = {"use_evo_norm": use_evo_norm, "evo_norm_group_size": evo_norm_group_size, "epsilon": bn_epsilon}
@@ -379,53 +380,64 @@ def AotNet(
     return model
 
 
+@register_model
 def AotNet50(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 4, 6, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [1, 2, 2, strides]
     return AotNet(**locals(), model_name=kwargs.pop("model_name", "aotnet50"), **kwargs)
 
 
+@register_model
 def AotNet101(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 4, 23, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [1, 2, 2, strides]
     return AotNet(**locals(), model_name=kwargs.pop("model_name", "aotnet101"), **kwargs)
 
 
+@register_model
 def AotNet152(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 8, 36, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [1, 2, 2, strides]
     return AotNet(**locals(), model_name=kwargs.pop("model_name", "aotnet152"), **kwargs)
 
 
+@register_model
 def AotNet200(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 24, 36, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [1, 2, 2, strides]
     return AotNet(**locals(), model_name=kwargs.pop("model_name", "aotnet200"), **kwargs)
 
 
-def AotNetV2(num_blocks, preact=True, strides=1, strides_first=False, **kwargs):
+""" AotNetV2 """
+
+
+def AotNetV2(num_blocks=[3, 4, 6, 3], preact=True, strides=1, strides_first=False, **kwargs):
     strides = strides if isinstance(strides, (list, tuple)) else [2, 2, 2, strides]
     return AotNet(num_blocks, preact=preact, strides=strides, strides_first=strides_first, **kwargs)
 
 
+@register_model
 def AotNet50V2(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 4, 6, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [2, 2, 2, strides]
     return AotNetV2(**locals(), model_name=kwargs.pop("model_name", "aotnet50v2"), **kwargs)
 
 
+@register_model
 def AotNet101V2(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 4, 23, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [2, 2, 2, strides]
     return AotNetV2(**locals(), model_name=kwargs.pop("model_name", "aotnet101v2"), **kwargs)
 
 
+@register_model
 def AotNet152V2(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 8, 36, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [2, 2, 2, strides]
     return AotNetV2(**locals(), model_name=kwargs.pop("model_name", "aotnet152v2"), **kwargs)
 
 
+@register_model
 def AotNet200V2(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", strides=2, **kwargs):
     num_blocks = [3, 24, 36, 3]
     strides = strides if isinstance(strides, (list, tuple)) else [2, 2, 2, strides]
