@@ -2,6 +2,7 @@ import numpy as np
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, functional, initializers, image_data_format
 from keras_cv_attention_models.aotnet import AotNet
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import conv2d_no_bias, scaled_dot_product_attention, qkv_to_multi_head_channels_last_format
 from keras_cv_attention_models.download_and_load import reload_model_weights
 
@@ -23,7 +24,7 @@ class RelativePositionalEmbedding(layers.Layer):
         self.position_height = position_height
         self.position_width = position_width if position_width > 0 else position_height
         self.use_absolute_pos = use_absolute_pos
-        self.dynamic_shape = dynamic_shape
+        self.dynamic_shape = dynamic_shape  # Deprecated
 
     def build(self, input_shape):
         _, num_heads, height, width, key_dim = input_shape
@@ -31,6 +32,7 @@ class RelativePositionalEmbedding(layers.Layer):
         self.position_width = self.position_width if self.position_width > width else width
         self.key_dim = key_dim
         stddev = key_dim**-0.5
+        self.num_heads, self.input_height, self.input_width = num_heads, height, width
 
         if self.use_absolute_pos:
             hh_shape = (key_dim, self.position_height)
@@ -42,7 +44,7 @@ class RelativePositionalEmbedding(layers.Layer):
         initializer = initializers.random_normal(stddev=stddev)
         self.pos_emb_h = self.add_weight(name="r_height", shape=hh_shape, initializer=initializer, trainable=True)
         self.pos_emb_w = self.add_weight(name="r_width", shape=ww_shape, initializer=initializer, trainable=True)
-        self.input_height, self.input_width = height, width
+
         super().build(input_shape)
 
     def get_config(self):
@@ -57,19 +59,22 @@ class RelativePositionalEmbedding(layers.Layer):
         )
         return base_config
 
-    def rel_to_abs(self, rel_pos):
+    def rel_to_abs(self, rel_pos, is_height):
         """
         Converts relative indexing to absolute.
         Input: [bs+heads, height, width, 2 * pos_dim - 1]
         Output: [bs+heads, height, width, pos_dim]
         """
-        bs_heads, hh, ww, dim = rel_pos.shape  # [bs+heads, height, width, 2 * width - 1]
-        pos_dim = (dim + 1) // 2
+        pos_dim = self.position_height if is_height else self.position_width  # Use static values
+        num_blocks = self.input_height if is_height else self.input_width  # Use static values
+
+        # pos_dim = (dim + 1) // 2
         if pos_dim == 1:
             return rel_pos
-        if ww == 1:
+        if num_blocks == 1:
             return rel_pos[:, :, :, -pos_dim:]
-        full_rank_gap = pos_dim - ww
+        _, hh, ww, dim = rel_pos.shape  # [bs+heads, height, width, 2 * width - 1]
+        full_rank_gap = pos_dim - num_blocks
         # [bs+heads, height, width * (2 * pos_dim - 1)] --> [bs+heads, height, width * (2 * pos_dim - 1) - width]
         flat_x = functional.reshape(rel_pos, [-1, hh, ww * dim])[:, :, ww - 1 : -1]
         # [bs+heads, height, width, 2 * (pos_dim - 1)] --> [bs+heads, height, width, pos_dim]
@@ -77,18 +82,18 @@ class RelativePositionalEmbedding(layers.Layer):
         return functional.reshape(flat_x, [-1, hh, ww, 2 * (pos_dim - 1)])[:, :, :, full_rank_gap : pos_dim + full_rank_gap]
 
     def relative_logits(self, inputs):
-        bs, heads, hh, ww, cc = inputs.shape  # e.g.: [1, 4, 14, 16, 128]
-        inputs = functional.reshape(inputs, [-1, hh, ww, cc])  # Merge bs and heads, for supporting TFLite conversion
+        # bs, heads, hh, ww, cc = inputs.shape  # e.g.: [1, 4, 14, 16, 128]
+        inputs = functional.reshape(inputs, [-1, self.input_height, self.input_width, self.key_dim])  # Merge bs and heads, for supporting TFLite conversion
         rel_logits_w = functional.matmul(inputs, self.pos_emb_w)  # [4, 14, 16, 31], 2 * 16 - 1 == 31
-        rel_logits_w = self.rel_to_abs(rel_logits_w)  # [4, 14, 16, 16]
+        rel_logits_w = self.rel_to_abs(rel_logits_w, is_height=False)  # [4, 14, 16, 16]
 
         query_h = functional.transpose(inputs, [0, 2, 1, 3])  # [4, 16, 14, 128], [bs+heads, ww, hh, dims], Exchange `ww` and `hh`
         rel_logits_h = functional.matmul(query_h, self.pos_emb_h)  # [4, 16, 14, 27], 2 * 14 - 1 == 27
-        rel_logits_h = self.rel_to_abs(rel_logits_h)  # [4, 16, 14, 14]
+        rel_logits_h = self.rel_to_abs(rel_logits_h, is_height=True)  # [4, 16, 14, 14]
         rel_logits_h = functional.transpose(rel_logits_h, [0, 2, 1, 3])  # [4, 14, 16, 14], transpose back
 
         logits = functional.expand_dims(rel_logits_w, axis=-2) + functional.expand_dims(rel_logits_h, axis=-1)  # [4, 14, 16, 14, 16]
-        return functional.reshape(logits, [-1, heads, hh, ww, self.position_height, self.position_width])  # [1, 4, 14, 16, 14, 16]
+        return functional.reshape(logits, [-1, self.num_heads, self.input_height, self.input_width, self.position_height, self.position_width])
 
     def absolute_logits(self, inputs):
         # pos_emb = tf.expand_dims(self.pos_emb_w, -2) + tf.expand_dims(self.pos_emb_h, -1)
@@ -127,8 +132,10 @@ class RelativePositionalEmbedding(layers.Layer):
     def show_pos_emb(self, base_size=4):
         import matplotlib.pyplot as plt
 
-        pos_emb_h = self.pos_emb_h.detach().numpy() if hasattr(self.pos_emb_h, "detach") else self.pos_emb_h.numpy()
-        pos_emb_w = self.pos_emb_w.detach().numpy() if hasattr(self.pos_emb_w, "detach") else self.pos_emb_w.numpy()
+        pos_emb_h = self.pos_emb_h.detach() if hasattr(self.pos_emb_h, "detach") else self.pos_emb_h
+        pos_emb_h = self.pos_emb_h.numpy() if hasattr(self.pos_emb_h, "numpy") else np.array(self.pos_emb_h)
+        pos_emb_w = self.pos_emb_w.detach() if hasattr(self.pos_emb_w, "detach") else self.pos_emb_w
+        pos_emb_w = self.pos_emb_w.numpy() if hasattr(self.pos_emb_w, "numpy") else np.array(self.pos_emb_w)
 
         fig, axes = plt.subplots(1, 3, figsize=(base_size * 3, base_size * 1))
         axes[0].imshow(pos_emb_h)
@@ -172,28 +179,6 @@ def mhsa_with_relative_position_embedding(
     out = scaled_dot_product_attention(query, key, value, output_shape, pos_emb_func, out_weight, out_bias, dropout=attn_dropout, name=name)
     return out if data_format == "channels_last" else layers.Permute([3, 1, 2])(out)
 
-    # query *= qk_scale
-    # [batch, num_heads, hh * ww, hh * ww]
-    # attention_scores = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([query, key]) * qk_scale
-    # pos_emb = functional.reshape(pos_emb, [-1, *attention_scores.shape[1:]])
-    # attention_scores = keras.layers.Add()([attention_scores, pos_emb])
-    # # attention_scores = tf.nn.softmax(attention_scores, axis=-1)
-    # attention_scores = keras.layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
-    #
-    # if attn_dropout > 0:
-    #     attention_scores = keras.layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores)
-    # # value = [batch, num_heads, hh * ww, vv_dim]
-    # # attention_output = tf.matmul(attention_scores, value)  # [batch, num_heads, hh * ww, vv_dim]
-    # attention_output = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([attention_scores, value])
-    # attention_output = tf.transpose(attention_output, perm=[0, 2, 1, 3])
-    # attention_output = tf.reshape(attention_output, [-1, inputs.shape[1], inputs.shape[2], num_heads * vv_dim])
-    # # print(f">>>> {attention_output.shape = }, {attention_scores.shape = }")
-    #
-    # if out_weight:
-    #     # [batch, hh, ww, num_heads * vv_dim] * [num_heads * vv_dim, out] --> [batch, hh, ww, out]
-    #     attention_output = keras.layers.Dense(out_shape, use_bias=out_bias, name=name and name + "output")(attention_output)
-    # return attention_output
-
 
 def BotNet(input_shape=(224, 224, 3), strides=1, pretrained="imagenet", **kwargs):
     attn_types = [None, None, None, "bot"]
@@ -205,21 +190,25 @@ def BotNet(input_shape=(224, 224, 3), strides=1, pretrained="imagenet", **kwargs
     return model
 
 
+@register_model
 def BotNet50(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", strides=1, **kwargs):
     num_blocks = [3, 4, 6, 3]
     return BotNet(**locals(), model_name="botnet50", **kwargs)
 
 
+@register_model
 def BotNet101(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained=None, strides=1, **kwargs):
     num_blocks = [3, 4, 23, 3]
     return BotNet(**locals(), model_name="botnet101", **kwargs)
 
 
+@register_model
 def BotNet152(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained=None, strides=1, **kwargs):
     num_blocks = [3, 8, 36, 3]
     return BotNet(**locals(), model_name="botnet152", **kwargs)
 
 
+@register_model
 def BotNet26T(input_shape=(256, 256, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 2, 2, 2]
     attn_types = [None, None, [None, "bot"], "bot"]
@@ -231,6 +220,7 @@ def BotNet26T(input_shape=(256, 256, 3), num_classes=1000, activation="relu", cl
     return model
 
 
+@register_model
 def BotNextECA26T(input_shape=(256, 256, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 2, 2, 2]
     attn_types = [None, None, [None, "bot"], "bot"]
@@ -243,6 +233,7 @@ def BotNextECA26T(input_shape=(256, 256, 3), num_classes=1000, activation="swish
     return model
 
 
+@register_model
 def BotNetSE33T(input_shape=(256, 256, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     stem_type = "tiered"
     stem_last_strides = 2
