@@ -1,7 +1,9 @@
 import os
+import numpy as np
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models, functional, callbacks
-from keras_cv_attention_models.coco import anchors_func
+from keras_cv_attention_models.coco import anchors_func, info
+from keras_cv_attention_models.models import no_grad_if_torch
 from tqdm import tqdm
 
 
@@ -35,6 +37,7 @@ class DecodePredictions(layers.Layer):
         anchor_scale="auto",
         aspect_ratios=(1, 2, 0.5),
         num_scales=3,
+        regression_len=4,  # bbox output len, typical value is 4, for yolov8 reg_max=16 -> regression_len = 16 * 4 == 64
         score_threshold=0.3,  # decode parameter, can be set new value in `self.call`
         iou_or_sigma=0.5,  # decode parameter, can be set new value in `self.call`
         max_output_size=100,  # decode parameter, can be set new value in `self.call`
@@ -42,20 +45,21 @@ class DecodePredictions(layers.Layer):
         mode="global",  # decode parameter, can be set new value in `self.call`
         topk=0,  # decode parameter, can be set new value in `self.call`
         use_static_output=False,  # Set to True if using this as an actual layer, especially for converting tflite
+        use_sigmoid_on_score=False,  # wether applying sigmoid on score outputs. Set True if model is built using `classifier_activation=None`
         **kwargs,
     ):
         super().__init__(**kwargs)
 
         self.pyramid_levels = list(range(min(pyramid_levels), max(pyramid_levels) + 1))
         use_object_scores, num_anchors, anchor_scale = anchors_func.get_anchors_mode_parameters(anchors_mode, use_object_scores, "auto", anchor_scale)
-        self.aspect_ratios, self.num_scales = aspect_ratios, num_scales
+        self.regression_len, self.aspect_ratios, self.num_scales = regression_len, aspect_ratios, num_scales
         self.anchors_mode, self.use_object_scores, self.anchor_scale = anchors_mode, use_object_scores, anchor_scale  # num_anchors not using
         if input_shape is not None and (isinstance(input_shape, (list, tuple)) and input_shape[1] is not None):
             self.__init_anchor__(input_shape)
         else:
             self.anchors = None
         self.__input_shape__ = input_shape
-        self.use_static_output = use_static_output
+        self.use_static_output, self.use_sigmoid_on_score = use_static_output, use_sigmoid_on_score
         self.nms_kwargs = {
             "score_threshold": score_threshold,
             "iou_or_sigma": iou_or_sigma,
@@ -67,17 +71,19 @@ class DecodePredictions(layers.Layer):
         super().build(input_shape)
 
     def __init_anchor__(self, input_shape):
-        if isinstance(input_shape, (list, tuple)):
+        if isinstance(input_shape, (list, tuple)) and len(input_shape) > 2:
             # input_shape = input_shape[:2] if backend.image_data_format() == "channels_last" else input_shape[-2:]
             channel_axis, channel_dim = min(enumerate(input_shape), key=lambda xx: xx[1])  # Assume the smallest value is the channel dimension
             input_shape = [dim for axis, dim in enumerate(input_shape) if axis != channel_axis]
-        else:
+        elif isinstance(input_shape, int):
             input_shape = (input_shape, input_shape)
 
         if self.anchors_mode == anchors_func.ANCHOR_FREE_MODE:
             self.anchors = anchors_func.get_anchor_free_anchors(input_shape, self.pyramid_levels)
         elif self.anchors_mode == anchors_func.YOLOR_MODE:
             self.anchors = anchors_func.get_yolor_anchors(input_shape, self.pyramid_levels)
+        elif self.anchors_mode == anchors_func.YOLOV8_MODE:
+            self.anchors = anchors_func.get_anchor_free_anchors(input_shape, self.pyramid_levels, grid_zero_start=False)
         else:
             grid_zero_start = False
             self.anchors = anchors_func.get_anchors(input_shape, self.pyramid_levels, self.aspect_ratios, self.num_scales, self.anchor_scale, grid_zero_start)
@@ -86,10 +92,10 @@ class DecodePredictions(layers.Layer):
 
     def __topk_class_boxes_single__(self, pred, topk=5000):
         # https://github.com/google/automl/tree/master/efficientdet/tf2/postprocess.py#L82
-        bbox_outputs, class_outputs = pred[:, :4], pred[:, 4:]
+        bbox_outputs, class_outputs = pred[:, : self.regression_len], pred[:, self.regression_len :]
         num_classes = class_outputs.shape[-1]
         class_outputs_flatten = functional.reshape(class_outputs, -1)
-        topk = class_outputs_flatten.shape[0] if topk == -1 else topk  # select all if -1
+        topk = class_outputs_flatten.shape[0] if topk == -1 else min(topk, class_outputs_flatten.shape[0])  # select all if -1
         _, class_topk_indices = functional.top_k(class_outputs_flatten, k=topk, sorted=False)
         # get original indices for class_outputs, original_indices_hh -> picking indices, original_indices_ww -> picked labels
         original_indices_hh, original_indices_ww = class_topk_indices // num_classes, class_topk_indices % num_classes
@@ -153,19 +159,23 @@ class DecodePredictions(layers.Layer):
             if self.use_object_scores:
                 ccs = ccs * functional.gather(object_scores, picking_indices)
         else:
-            bbs, ccs, labels = pred[:, :4], functional.reduce_max(pred[:, 4:], axis=-1), functional.argmax(pred[:, 4:], axis=-1)
+            bbs, scores = pred[:, : self.regression_len], pred[:, self.regression_len :]
+            ccs, labels = functional.reduce_max(scores, axis=-1), functional.argmax(scores, axis=-1)
             anchors = self.anchors
             if self.use_object_scores:
                 ccs = ccs * object_scores
+        ccs = functional.sigmoid(ccs) if self.use_sigmoid_on_score else ccs
 
         # print(f"{bbs.shape = }, {anchors.shape = }")
-        bbs_decoded = anchors_func.decode_bboxes(bbs, anchors)
+        bbs_decoded = anchors_func.decode_bboxes(bbs, anchors, regression_len=self.regression_len)
         iou_threshold, soft_nms_sigma = (1.0, iou_or_sigma / 2) if method.lower() == "gaussian" else (iou_or_sigma, 0.0)
 
         if mode == "per_class":
             bboxs, lables, confidences = self.__nms_per_class__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
-        else:
+        elif mode == "global":
             bboxs, lables, confidences = self.__nms_global__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
+        else:
+            bboxs, lables, confidences = bbs_decoded, labels, ccs  # Return raw decoded data for testing
 
         return self.__to_static__(bboxs, lables, confidences, max_output_size) if self.use_static_output else (bboxs, lables, confidences)
 
@@ -209,29 +219,44 @@ class DecodePredictions(layers.Layer):
         return config
 
 
+""" COCO Evaluation """
+
+
 def scale_bboxes_back_single(bboxes, image_shape, scale, pad_top, pad_left, target_shape):
     # height, width = target_shape[0] / scale, target_shape[1] / scale
     # bboxes *= [height, width, height, width]
     bboxes *= [target_shape[0], target_shape[1], target_shape[0], target_shape[1]]
     bboxes -= [pad_top, pad_left, pad_top, pad_left]
     bboxes /= scale
-    bboxes = functional.clip_by_value(bboxes, 0, clip_value_max=[image_shape[0], image_shape[1], image_shape[0], image_shape[1]])
+    clip_value_max = functional.convert_to_tensor([image_shape[0], image_shape[1], image_shape[0], image_shape[1]], dtype="float32")
+    bboxes = functional.clip_by_value(bboxes, 0, clip_value_max=clip_value_max)
     # [top, left, bottom, right] -> [left, top, width, height]
     bboxes = functional.stack([bboxes[:, 1], bboxes[:, 0], bboxes[:, 3] - bboxes[:, 1], bboxes[:, 2] - bboxes[:, 0]], axis=-1)
     return bboxes
 
 
 def image_process(image, target_shape, mean, std, resize_method="bilinear", resize_antialias=False, use_bgr_input=False, letterbox_pad=-1):
-    from keras_cv_attention_models.coco import data
+    if backend.is_tensorflow_backend:
+        from keras_cv_attention_models.coco.tf_data import tf_imread as imread, aspect_aware_resize_and_crop_image
+    else:
+        import cv2
+        from keras_cv_attention_models.coco.torch_data import aspect_aware_resize_and_crop_image
 
-    if len(image.shape) < 2:
-        image = data.tf_imread(image)  # it's image path
-    original_image_shape = functional.shape(image)[:2]
-    image = functional.cast(image, "float32")
-    image = (image - mean) / std  # automl behavior: rescale -> resize
-    image, scale, pad_top, pad_left = data.aspect_aware_resize_and_crop_image(
+        imread = lambda image_path: cv2.imread(image_path)[:, :, ::-1]  # BGR -> RGB
+
+    if isinstance(image, str) or len(image.shape) < 2:
+        image = imread(image)  # it's image path
+
+    if backend.is_tensorflow_backend:
+        original_image_shape = functional.shape(image)[:2]
+        image = functional.cast(image, "float32")
+    else:
+        original_image_shape, image = image.shape[:2], image.astype("float32")
+
+    image, scale, pad_top, pad_left = aspect_aware_resize_and_crop_image(
         image, target_shape, letterbox_pad=letterbox_pad, method=resize_method, antialias=resize_antialias
     )
+    image = (image - mean) / std  # automl behavior: rescale -> resize
     if use_bgr_input:
         image = image[:, :, ::-1]
     return image, scale, pad_top, pad_left, original_image_shape
@@ -247,27 +272,56 @@ def init_eval_dataset(
     letterbox_pad=-1,
     use_bgr_input=False,
 ):
-    import tensorflow_datasets as tfds
-    from keras_cv_attention_models.coco import data
+    if backend.is_tensorflow_backend:
+        from keras_cv_attention_models.coco import tf_data
 
-    dataset = data.detection_dataset_from_custom_json(data_name) if data_name.endswith(".json") else tfds.load(data_name)
-    ds = dataset.get("validation", dataset.get("test", None))
+        if data_name.endswith(".json"):
+            dataset, _, num_classes = tf_data.detection_dataset_from_custom_json(data_name, with_info=True)
+        else:
+            import tensorflow_datasets as tfds
 
-    mean, std = data.init_mean_std_by_rescale_mode(rescale_mode)
-    __image_process__ = lambda image: image_process(image, input_shape, mean, std, resize_method, resize_antialias, use_bgr_input, letterbox_pad)
-    # ds: [resized_image, scale, pad_top, pad_left, original_image_shape, image_id]
-    ds = ds.map(lambda datapoint: (*__image_process__(datapoint["image"]), datapoint.get("image/id", datapoint["image"])))
-    ds = ds.batch(batch_size)
-    return ds
+            dataset, info = tfds.load(data_name, with_info=True)
+            num_classes = info.features["objects"]["label"].num_classes
+
+        ds = dataset.get("validation", dataset.get("test", None))
+
+        mean, std = tf_data.init_mean_std_by_rescale_mode(rescale_mode)
+        __image_process__ = lambda image: image_process(image, input_shape, mean, std, resize_method, resize_antialias, use_bgr_input, letterbox_pad)
+        # ds: [resized_image, scale, pad_top, pad_left, original_image_shape, image_id]
+        ds = ds.map(lambda datapoint: (*__image_process__(datapoint["image"]), datapoint.get("image/id", datapoint["image"])))
+        ds = ds.batch(batch_size)
+        return ds, num_classes
+    else:
+        import torch
+        from torch.utils.data import Dataset, DataLoader
+        from keras_cv_attention_models.coco import torch_data
+
+        _, test, total_images, num_classes = torch_data.load_from_custom_json(data_name)
+        mean, std = torch_data.init_mean_std_by_rescale_mode(rescale_mode, convert_to_image_data_format=False)
+
+        class EvalDataset(Dataset):
+            def __len__(self):
+                return len(test)
+
+            def __getitem__(self, index):
+                image_path = test[index]["image"]
+                image, scale, pad_top, pad_left, original_image_shape = image_process(
+                    image_path, input_shape, mean, std, resize_method, resize_antialias, use_bgr_input, letterbox_pad
+                )
+                image = torch.from_numpy(image).permute([2, 0, 1]).contiguous()
+                return image, scale, pad_top, pad_left, torch.as_tensor(original_image_shape), image_path
+
+        ds = DataLoader(EvalDataset(), batch_size=batch_size, shuffle=False, num_workers=8, pin_memory=True, sampler=None, drop_last=False)
+        # ds.element_spec = next(iter(ds))
+        return ds, num_classes
 
 
-def model_detection_and_decode(model, eval_dataset, pred_decoder, nms_kwargs={}, is_coco=True, image_id_map=None):
-    from keras_cv_attention_models.coco import data
-
-    target_shape = (eval_dataset.element_spec[0].shape[1], eval_dataset.element_spec[0].shape[2])
-    num_classes = model.output_shape[-1] - 4
+def model_detection_and_decode(model, eval_dataset, pred_decoder, nms_kwargs={}, is_coco=True, image_id_map=None, num_classes=80):
+    sample_image = next(iter(eval_dataset))[0]
+    target_shape = sample_image.shape[1:-1] if backend.image_data_format() == "channels_last" else sample_image.shape[2:]
+    # num_classes = model.output_shape[-1] - 4
     if is_coco:
-        to_91_labels = (lambda label: label + 1) if num_classes >= 90 else (lambda label: data.COCO_80_to_90_LABEL_DICT[label] + 1)
+        to_91_labels = (lambda label: label + 1) if num_classes >= 90 else (lambda label: info.COCO_80_to_90_LABEL_DICT[label] + 1)
     else:
         to_91_labels = lambda label: label
     # Format: [image_id, x, y, width, height, score, class]
@@ -275,20 +329,19 @@ def model_detection_and_decode(model, eval_dataset, pred_decoder, nms_kwargs={},
 
     results = []
     for images, scales, pad_tops, pad_lefts, original_image_shapes, image_ids in tqdm(eval_dataset):
-        preds = model(images)
-        preds = functional.cast(preds, "float32")
+        preds = model.predict(images).cpu().float() if backend.is_torch_backend else functional.cast(model(images), "float32")
         # decoded_preds: [[bboxes, labels, scores], [bboxes, labels, scores], ...]
         decoded_preds = pred_decoder(preds, **nms_kwargs)
 
         # Loop on batch
         for rr, image_shape, scale, pad_top, pad_left, image_id in zip(decoded_preds, original_image_shapes, scales, pad_tops, pad_lefts, image_ids):
             bboxes, labels, scores = rr
-            image_id, bboxes, labels, scores = image_id.numpy(), bboxes.numpy(), labels.numpy(), scores.numpy()
+            image_id, bboxes, labels, scores = np.array(image_id).item(), bboxes.numpy(), labels.numpy(), scores.numpy()
             if image_id_map is not None:
                 image_id = image_id_map[image_id.decode() if isinstance(image_id, bytes) else image_id]
             bboxes = scale_bboxes_back_single(bboxes, image_shape, scale, pad_top, pad_left, target_shape).numpy()
             results.extend([to_coco_eval_single(image_id, bb, cc, ss) for bb, cc, ss in zip(bboxes, labels, scores)])  # Loop on prediction results
-    return functional.convert_to_tensor(results).numpy()
+    return np.array(results)
 
 
 class COCOEvaluation:
@@ -296,8 +349,9 @@ class COCOEvaluation:
         from pycocotools.coco import COCO
 
         if annotations is None:
-            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/efficientdet/coco_annotations_instances_val2017.json"
-            annotations = backend.get_file(origin=url)
+            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/assets/coco_annotations_instances_val2017.json"
+            file_hash = "b681580a54b900b3cb44022fd1102ad5"
+            annotations = backend.get_file(origin=url, file_hash=file_hash)
 
         if isinstance(annotations, dict):  # json already loaded as dict
             coco_gt = COCO()
@@ -340,9 +394,11 @@ def to_coco_annotation(json_path):
 
     # int conversion just in case key is str
     categories = {int(kk): vv for kk, vv in aa["indices_2_labels"].items()} if "indices_2_labels" in aa else {}
+    base_path = os.path.expanduser(aa["info"]["base_path"]) if "base_path" in aa.get("info", {}) and len(aa["info"]["base_path"]) > 0 else None
     annotations, images, image_id_map = [], [], {}
     for image_id, ii in enumerate(aa.get("validation", aa.get("test", []))):
-        width, height = Image.open(ii["image"]).size  # For decoding bboxes, not actually openning images
+        image_file = os.path.join(base_path, ii["image"]) if base_path else ii["image"]
+        width, height = Image.open(image_file).size  # For decoding bboxes, not actually openning images
         for bb, label in zip(ii["objects"]["bbox"], ii["objects"]["label"]):
             # bb [top, left, bottom, right] in [0, 1] -> [left, top, bbox_width, bbox_height] with actual coordinates
             top = bb[0] * height
@@ -356,19 +412,21 @@ def to_coco_annotation(json_path):
             annotations.append({"bbox": bb, "category_id": label, "image_id": image_id, "id": len(annotations), "iscrowd": 0, "area": area})
             if label not in categories:
                 categories[label] = str(len(categories))
-        images.append({"id": image_id, "file_name": ii["image"], "height": height, "width": width})
-        image_id_map[ii["image"]] = image_id
+        images.append({"id": image_id, "file_name": image_file, "height": height, "width": width})
+        image_id_map[image_file] = image_id
     categories = [{"id": kk, "name": vv} for kk, vv in categories.items()]
     return {"images": images, "annotations": annotations, "categories": categories}, image_id_map
 
 
-# Wrapper a callback for using in training
+""" Wrapper a callback for using in training """
+
+
 class COCOEvalCallback(callbacks.Callback):
     """
     Basic test:
     >>> from keras_cv_attention_models import efficientdet, coco
     >>> model = efficientdet.EfficientDetD0()
-    >>> ee = coco.eval_func.COCOEvalCallback(batch_size=4, model_basic_save_name='test', rescale_mode='raw', anchors_mode="anchor_free")
+    >>> ee = coco.eval_func.COCOEvalCallback(batch_size=4)
     >>> ee.model = model
     >>> ee.on_epoch_end()
     """
@@ -386,8 +444,8 @@ class COCOEvalCallback(callbacks.Callback):
         nms_score_threshold=0.001,  # [model_detection_and_decode parameters]
         nms_iou_or_sigma=0.5,
         nms_max_output_size=100,
-        nms_method="gaussian",
-        nms_mode="per_class",
+        nms_method="gaussian",  # gaussian or hard
+        nms_mode="per_class",  # per_class or global
         nms_topk=5000,
         anchors_mode="auto",  # [model anchors related parameters]
         anchor_scale=4,  # Init anchors for model prediction. "auto" means 1 if (anchors_mode=="anchor_free" or anchors_mode=="yolor"), else 4
@@ -438,37 +496,45 @@ class COCOEvalCallback(callbacks.Callback):
         self.built = False
 
     def build(self, input_shape, output_shape):
-        input_shape = (int(input_shape[1]), int(input_shape[2]))
-        self.eval_dataset = init_eval_dataset(input_shape=input_shape, **self.dataset_kwargs)
+        import re
+
+        input_shape = (
+            (int(input_shape[1]), int(input_shape[2])) if backend.image_data_format() == "channels_last" else (int(input_shape[2]), int(input_shape[3]))
+        )
+        self.eval_dataset, self.num_classes = init_eval_dataset(input_shape=input_shape, **self.dataset_kwargs)
+        print("\n>>>> [COCOEvalCallback] self.dataset_kwargs:", self.dataset_kwargs)
+        regression_len = (output_shape[-1] - self.num_classes) // 4 * 4
+
         if self.anchors_mode is None or self.anchors_mode == "auto":
-            self.anchors_mode, num_anchors = anchors_func.get_anchors_mode_by_anchors(input_shape, total_anchors=output_shape[1])
+            self.anchors_mode, num_anchors = anchors_func.get_anchors_mode_by_anchors(input_shape, total_anchors=output_shape[1], regression_len=regression_len)
         elif self.anchors_mode == anchors_func.EFFICIENTDET_MODE:
             num_anchors = self.efficient_det_num_anchors
         else:
             num_anchors = anchors_func.NUM_ANCHORS.get(self.anchors_mode, 9)
         pyramid_levels = anchors_func.get_pyramid_levels_by_anchors(input_shape, total_anchors=output_shape[1], num_anchors=num_anchors)
-        print("\n>>>> [COCOEvalCallback] input_shape: {}, pyramid_levels: {}, anchors_mode: {}".format(input_shape, pyramid_levels, self.anchors_mode))
+        print(">>>> [COCOEvalCallback] input_shape: {}, pyramid_levels: {}, anchors_mode: {}".format(input_shape, pyramid_levels, self.anchors_mode))
         # print(">>>>", self.dataset_kwargs)
         # print(">>>>", self.nms_kwargs)
 
-        self.pred_decoder = DecodePredictions(input_shape, pyramid_levels, self.anchors_mode, **self.anchor_kwargs)
+        use_sigmoid_on_score = not any([ii.name.endswith("_sigmoid") for ii in self.model.layers[-50:]])
+        print(">>>> use_sigmoid_on_score:", use_sigmoid_on_score)
+        self.pred_decoder = DecodePredictions(
+            input_shape, pyramid_levels, self.anchors_mode, regression_len=regression_len, use_sigmoid_on_score=use_sigmoid_on_score, **self.anchor_kwargs
+        )
 
         # Training saving best
         if self.model_basic_save_name is not None:
-            self.monitor_save = os.path.join(self.save_path, self.model_basic_save_name + "_epoch_{}_" + self.item_key + "_{}.h5")
-            self.monitor_save_re = self.monitor_save.format("*", "*")
+            monitor_save_name = self.model_basic_save_name + "_epoch_{}_" + self.item_key + "_{}.h5"
+            self.monitor_save_re = re.compile(monitor_save_name.format(r"\d*", r"[\d\.]*"))
+            self.monitor_save = os.path.join(self.save_path, monitor_save_name)
             self.is_better = lambda cur, pre: cur >= pre
             self.pre_best = -1e5
 
         self.coco_evaluation = COCOEvaluation(self.annotation_file)
         self.built = True
 
+    @no_grad_if_torch
     def on_epoch_end(self, epoch=0, logs=None):
-        if backend.is_tensorflow_backend:
-            from tensorflow.io.gfile import glob
-        else:
-            from glob2 import glob
-
         if not self.built:
             if self.dataset_kwargs["rescale_mode"] == "auto":
                 self.dataset_kwargs["rescale_mode"] = getattr(self.model, "rescale_mode", "torch")
@@ -479,30 +545,27 @@ class COCOEvalCallback(callbacks.Callback):
 
         # pred_decoder = self.model.decode_predictions
         eval_dataset = self.eval_dataset.take(self.take_samples) if self.take_samples > 0 else self.eval_dataset
-        detection_results = model_detection_and_decode(self.model, eval_dataset, self.pred_decoder, self.nms_kwargs, self.is_coco, self.image_id_map)
-        try:
-            coco_eval = self.coco_evaluation(detection_results)
-        except:
-            print(">>>> Error in running coco_evaluation")
-            coco_eval = None
-            data_name = self.data_name.replace("/", "_")
-            self.save_json = "{}_{}_detection_results_error.json".format(self.model.name, data_name) if self.save_json is None else self.save_json
+        detection_results = model_detection_and_decode(
+            self.model, eval_dataset, self.pred_decoder, self.nms_kwargs, self.is_coco, self.image_id_map, self.num_classes
+        )
+        coco_eval = None if len(detection_results) == 0 else self.coco_evaluation(detection_results)
 
         if self.save_json is not None:
             to_coco_json(detection_results, self.save_json)
             print(">>>> Detection results saved to:", self.save_json)
 
         if hasattr(self.model, "history") and hasattr(self.model.history, "history"):
-            self.model.history.history.setdefault(self.item_key, []).append(coco_eval.stats.tolist())
+            self.model.history.history.setdefault(self.item_key, []).append(([0] * 12) if coco_eval is None else coco_eval.stats.tolist())
 
         # Training save best
         cur_ap = coco_eval.stats[0] if coco_eval is not None else 0
         if self.model_basic_save_name is not None and self.is_better(cur_ap, self.pre_best):
             self.pre_best = cur_ap
-            pre_monitor_saves = glob(self.monitor_save_re)
+            # pre_monitor_saves = glob(self.monitor_save_re)
+            pre_monitor_saves = [ii for ii in os.listdir(self.save_path) if self.monitor_save_re.match(ii)]
             # tf.print(">>>> pre_monitor_saves:", pre_monitor_saves)
             if len(pre_monitor_saves) != 0:
-                os.remove(pre_monitor_saves[0])
+                os.remove(os.path.join(self.save_path, pre_monitor_saves[0]))
             monitor_save = self.monitor_save.format(epoch + 1, "{:.4f}".format(cur_ap))
             print("\n>>>> Save best to:", monitor_save)
             if self.model is not None:
