@@ -6,7 +6,6 @@ BATCH_NORM_DECAY = 0.9
 BATCH_NORM_EPSILON = 1e-5
 TF_BATCH_NORM_EPSILON = 0.001
 LAYER_NORM_EPSILON = 1e-5
-CONV_KERNEL_INITIALIZER = "glorot_uniform"
 
 
 """ Wrapper for default parameters """
@@ -44,6 +43,13 @@ def phish(inputs):
     return inputs * functional.tanh(functional.gelu(inputs))
 
 
+def gelu_quick(inputs):
+    """https://github.com/huggingface/transformers/blob/main/src/transformers/activations.py#L90-L98
+    Applies GELU approximation that is fast but somewhat inaccurate. See: https://github.com/hendrycks/GELUs
+    """
+    return inputs * functional.sigmoid(1.702 * inputs)
+
+
 def gelu_linear(inputs):
     """
     >>> from keras_cv_attention_models.common_layers import gelu_linear
@@ -59,7 +65,7 @@ def gelu_linear(inputs):
 
     erf = inputs_abs * -0.7071
     erf = functional.relu(erf + 1.769)
-    erf = erf ** 2 * -0.1444 + 0.5
+    erf = erf**2 * -0.1444 + 0.5
     return inputs * (erf * inputs_sign + 0.5)
 
 
@@ -72,6 +78,8 @@ def activation_by_name(inputs, activation="relu", name=None):
     activation_lower = activation.lower()
     if activation_lower == "hard_swish":
         return layers.Activation(activation=hard_swish, name=layer_name)(inputs)
+    if activation_lower == "leaky_relu":
+        return layers.LeakyReLU(name=layer_name)(inputs)
     elif activation_lower == "mish":
         return layers.Activation(activation=mish, name=layer_name)(inputs)
     elif activation_lower == "phish":
@@ -83,9 +91,11 @@ def activation_by_name(inputs, activation="relu", name=None):
         return layers.PReLU(shared_axes=shared_axes, alpha_initializer=initializers.Constant(0.25), name=layer_name)(inputs)
     elif activation_lower.startswith("gelu/app"):
         # gelu/approximate
-        return functional.gelu(inputs, approximate=True, name=layer_name)
+        return functional.gelu(inputs, approximate=True)
     elif activation_lower.startswith("gelu/linear"):
         return gelu_linear(inputs)
+    elif activation_lower.startswith("gelu/quick"):
+        return gelu_quick(inputs)
     elif activation_lower.startswith("leaky_relu/"):
         # leaky_relu with alpha parameter
         alpha = float(activation_lower.split("/")[-1])
@@ -264,59 +274,82 @@ def group_norm(inputs, groups=32, epsilon=BATCH_NORM_EPSILON, axis="auto", name=
     return GroupNormalization(groups=groups, axis=norm_axis, epsilon=epsilon, name=name and name + "group_norm")(inputs)
 
 
-def conv2d_no_bias(inputs, filters, kernel_size=1, strides=1, padding="VALID", use_bias=False, groups=1, use_torch_padding=True, name=None, **kwargs):
-    """Typical Conv2D with `use_bias` default as `False` and fixed padding"""
+def conv2d_no_bias(inputs, filters, kernel_size=1, strides=1, padding="valid", use_bias=False, groups=1, use_torch_padding=True, name=None, **kwargs):
+    """Typical Conv2D with `use_bias` default as `False` and fixed padding,
+    and torch initializer `uniform(-1/sqrt(k), 1/sqrt(k)), where k = weight.size(1) * prod(*kernel_size)`
+    """
     kernel_size = kernel_size if isinstance(kernel_size, (list, tuple)) else (kernel_size, kernel_size)
     if isinstance(padding, str):
-        padding = padding.upper()
-        pad = (kernel_size[0] // 2, kernel_size[1] // 2) if use_torch_padding and padding == "SAME" else (0, 0)
+        padding = padding.lower()
+        pad = (kernel_size[0] // 2, kernel_size[1] // 2) if use_torch_padding and padding == "same" else (0, 0)
     else:  # int or list or tuple with specific value
         pad = padding if isinstance(padding, (list, tuple)) else (padding, padding)
-        padding = "SAME" if max(pad) > 0 else "VALID"
+        padding = "same" if max(pad) > 0 else "valid"
 
-    if use_torch_padding and backend.is_tensorflow_backend and padding == "SAME":
-        inputs = layers.ZeroPadding2D(padding=pad, name=name and name + "pad")(inputs)
-        padding = "VALID"
+    if use_torch_padding and not backend.is_torch_backend and padding == "same":
+        inputs = layers.ZeroPadding2D(padding=pad, name=name and name + "pad")(inputs) if max(pad) > 0 else inputs
+        padding = "valid"
+
+    kernel_initializer = kwargs.get("kernel_initializer", None)
+    if kernel_initializer is None and not backend.is_torch_backend:
+        fan_in = 1 / (float(inputs.shape[-1] * kernel_size[0] * kernel_size[1]) ** 0.5)
+        kernel_initializer = initializers.RandomUniform(-fan_in, fan_in)
 
     groups = max(1, groups)
     return layers.Conv2D(
         filters,
         kernel_size,
         strides=strides,
-        padding="VALID" if padding == "VALID" else (pad if use_torch_padding else "SAME"),
+        padding="valid" if padding == "valid" else (pad if use_torch_padding else "same"),
         use_bias=use_bias,
         groups=groups,
-        kernel_initializer=CONV_KERNEL_INITIALIZER,
+        kernel_initializer=kernel_initializer,
         name=name and name + "conv",
         **kwargs,
     )(inputs)
 
 
-def depthwise_conv2d_no_bias(inputs, kernel_size, strides=1, padding="VALID", use_bias=False, use_torch_padding=True, name=None, **kwargs):
-    """Typical DepthwiseConv2D with `use_bias` default as `False` and fixed padding"""
+def depthwise_conv2d_no_bias(inputs, kernel_size, strides=1, padding="valid", use_bias=False, use_torch_padding=True, name=None, **kwargs):
+    """Typical DepthwiseConv2D with `use_bias` default as `False` and fixed padding
+    and torch initializer `uniform(-1/sqrt(k), 1/sqrt(k)), where k = weight.size(1) * prod(*kernel_size)`
+    """
     kernel_size = kernel_size if isinstance(kernel_size, (list, tuple)) else (kernel_size, kernel_size)
     if isinstance(padding, str):
-        padding = padding.upper()
-        pad = (kernel_size[0] // 2, kernel_size[1] // 2) if use_torch_padding and padding == "SAME" else (0, 0)
+        padding = padding.lower()
+        pad = (kernel_size[0] // 2, kernel_size[1] // 2) if use_torch_padding and padding == "same" else (0, 0)
     else:  # int or list or tuple with specific value
         pad = padding if isinstance(padding, (list, tuple)) else (padding, padding)
-        padding = "SAME" if max(pad) > 0 else "VALID"
+        padding = "same" if max(pad) > 0 else "valid"
 
-    if use_torch_padding and backend.is_tensorflow_backend and padding == "SAME":
-        inputs = layers.ZeroPadding2D(padding=pad, name=name and name + "pad")(inputs)
-        padding = "VALID"
+    if use_torch_padding and not backend.is_torch_backend and padding == "same":
+        inputs = layers.ZeroPadding2D(padding=pad, name=name and name + "pad")(inputs) if max(pad) > 0 else inputs
+        padding = "valid"
+
+    depthwise_initializer = kwargs.get("depthwise_initializer", None)
+    if depthwise_initializer is None and not backend.is_torch_backend:
+        fan_in = 1 / (float(inputs.shape[-1] * kernel_size[0] * kernel_size[1]) ** 0.5)
+        depthwise_initializer = initializers.RandomUniform(-fan_in, fan_in)
 
     return layers.DepthwiseConv2D(
         kernel_size,
         strides=strides,
-        padding="VALID" if padding == "VALID" else (pad if use_torch_padding else "SAME"),
+        padding="valid" if padding == "valid" else (pad if use_torch_padding else "same"),
         use_bias=use_bias,
-        depthwise_initializer=CONV_KERNEL_INITIALIZER,
+        depthwise_initializer=depthwise_initializer,
         name=name and name + "dw_conv",
         **kwargs,
     )(inputs)
 
 
+def dense_no_bias(inputs, units, use_bias=False, name=None, **kwargs):
+    """Typical Dense with `use_bias` default as `False`, and Torch Linear initializer `uniform(-1/sqrt(in_features), 1/sqrt(in_features))`"""
+    kernel_initializer = kwargs.get("kernel_initializer", None)
+    if kernel_initializer is None and not backend.is_torch_backend:
+        fan_in = 1 / (float(inputs.shape[-1]) ** 0.5)
+        kernel_initializer = initializers.RandomUniform(-fan_in, fan_in)
+    return layers.Dense(units, kernel_initializer=kernel_initializer, use_bias=use_bias, name=name, **kwargs)(inputs)
+
+
 """ Blocks """
 
 
@@ -347,7 +380,7 @@ def global_context_module(inputs, use_attn=True, ratio=0.25, divisor=1, activati
     reduction = make_divisible(filters * ratio, divisor, limit_round_down=0.0)
 
     if use_attn:
-        attn = layers.Conv2D(1, kernel_size=1, use_bias=use_bias, kernel_initializer=CONV_KERNEL_INITIALIZER, name=name and name + "attn_conv")(inputs)
+        attn = layers.Conv2D(1, kernel_size=1, use_bias=use_bias, name=name and name + "attn_conv")(inputs)
         attn = functional.reshape(attn, [-1, 1, height * width])  # [batch, height, width, 1] or [batch, 1, height, width] -> [batch, 1, height * width]
         attn = functional.softmax(attn, axis=-1)
         context = inputs if is_channels_last else functional.transpose(inputs, [0, 2, 3, 1])
@@ -377,14 +410,14 @@ def se_module(inputs, se_ratio=0.25, divisor=8, limit_round_down=0.9, activation
     # print(f"{filters = }, {se_ratio = }, {divisor = }, {reduction = }")
     se = functional.reduce_mean(inputs, [h_axis, w_axis], keepdims=True if use_conv else False)
     if use_conv:
-        se = layers.Conv2D(reduction, kernel_size=1, use_bias=use_bias, kernel_initializer=CONV_KERNEL_INITIALIZER, name=name and name + "1_conv")(se)
+        se = layers.Conv2D(reduction, kernel_size=1, use_bias=use_bias, name=name and name + "1_conv")(se)
     else:
-        se = layers.Dense(reduction, use_bias=use_bias, kernel_initializer=CONV_KERNEL_INITIALIZER, name=name and name + "1_dense")(se)
+        se = layers.Dense(reduction, use_bias=use_bias, name=name and name + "1_dense")(se)
     se = activation_by_name(se, activation=hidden_activation, name=name)
     if use_conv:
-        se = layers.Conv2D(filters, kernel_size=1, use_bias=use_bias, kernel_initializer=CONV_KERNEL_INITIALIZER, name=name and name + "2_conv")(se)
+        se = layers.Conv2D(filters, kernel_size=1, use_bias=use_bias, name=name and name + "2_conv")(se)
     else:
-        se = layers.Dense(filters, use_bias=use_bias, kernel_initializer=CONV_KERNEL_INITIALIZER, name=name and name + "2_dense")(se)
+        se = layers.Dense(filters, use_bias=use_bias, name=name and name + "2_dense")(se)
     se = activation_by_name(se, activation=output_activation, name=name)
     se = se if use_conv else functional.reshape(se, [-1, 1, 1, filters] if image_data_format() == "channels_last" else [-1, filters, 1, 1])
     return layers.Multiply(name=name and name + "out")([inputs, se])
@@ -405,7 +438,7 @@ def eca_module(inputs, gamma=2.0, beta=1.0, name=None, **kwargs):
     nn = functional.pad(nn, [[0, 0], [pad, pad]])
     nn = functional.expand_dims(nn, channel_axis)
 
-    nn = layers.Conv1D(1, kernel_size=kernel_size, strides=1, padding="VALID", use_bias=False, name=name and name + "conv1d")(nn)
+    nn = layers.Conv1D(1, kernel_size=kernel_size, strides=1, padding="valid", use_bias=False, name=name and name + "conv1d")(nn)
     nn = functional.squeeze(nn, axis=channel_axis)
     nn = activation_by_name(nn, activation="sigmoid", name=name)
     nn = nn[:, None, None] if image_data_format() == "channels_last" else nn[:, :, None, None]
@@ -415,8 +448,10 @@ def eca_module(inputs, gamma=2.0, beta=1.0, name=None, **kwargs):
 
 def drop_connect_rates_split(num_blocks, start=0.0, end=0.0):
     """split drop connect rate in range `(start, end)` according to `num_blocks`"""
-    drop_connect_rates = functional.split(functional.linspace(start, end, sum(num_blocks)), num_blocks)
-    return [ii.numpy().tolist() for ii in drop_connect_rates]
+    # drop_connect_rates = functional.split(functional.linspace(start, end, sum(num_blocks)), num_blocks)
+    cum_split = [sum(num_blocks[: id + 1]) for id, _ in enumerate(num_blocks[:-1])]
+    drop_connect_rates = np.split(np.linspace(start, end, sum(num_blocks)), cum_split)
+    return [ii.tolist() for ii in drop_connect_rates]
 
 
 def drop_block(inputs, drop_rate=0, name=None):
@@ -463,12 +498,12 @@ def __anti_alias_downsample_initializer__(weight_shape, dtype="float32"):
     return functional.convert_to_tensor(ww, dtype=dtype)
 
 
-def anti_alias_downsample(inputs, kernel_size=3, strides=2, padding="SAME", trainable=False, name=None):
+def anti_alias_downsample(inputs, kernel_size=3, strides=2, padding="same", trainable=False, name=None):
     """DepthwiseConv2D performing anti-aliasing downsample, arxiv: https://arxiv.org/pdf/1904.11486.pdf"""
     return layers.DepthwiseConv2D(
         kernel_size=kernel_size,
         strides=strides,
-        padding="SAME",
+        padding="same",
         use_bias=False,
         trainable=trainable,
         depthwise_initializer=__anti_alias_downsample_initializer__,
@@ -491,13 +526,13 @@ def make_divisible(vv, divisor=4, min_value=None, limit_round_down=0.9):
 def __unfold_filters_initializer__(weight_shape, dtype="float32"):
     kernel_size = weight_shape[0]
     kernel_out = kernel_size * kernel_size
-    ww = functional.reshape(functional.eye(kernel_out), [kernel_size, kernel_size, 1, kernel_out])
+    ww = np.reshape(np.eye(kernel_out, dtype="float32"), [kernel_size, kernel_size, 1, kernel_out])
     if len(weight_shape) == 5:  # Conv3D or Conv3DTranspose
-        ww = functional.expand_dims(ww, 2)
-    return ww
+        ww = np.expand_dims(ww, 2)
+    return functional.convert_to_tensor(ww)
 
 
-def fold_by_conv2d_transpose(patches, output_shape=None, kernel_size=3, strides=2, dilation_rate=1, padding="SAME", compressed="auto", name=None):
+def fold_by_conv2d_transpose(patches, output_shape=None, kernel_size=3, strides=2, dilation_rate=1, padding="same", compressed="auto", name=None):
     paded = kernel_size // 2 if padding else 0
     if compressed == "auto":
         compressed = True if len(patches.shape) == 4 else False
@@ -518,7 +553,7 @@ def fold_by_conv2d_transpose(patches, output_shape=None, kernel_size=3, strides=
         kernel_size=kernel_size,
         strides=strides,
         dilation_rate=dilation_rate,
-        padding="VALID",
+        padding="valid",
         output_padding=paded,
         use_bias=False,
         trainable=False,
@@ -538,14 +573,15 @@ def fold_by_conv2d_transpose(patches, output_shape=None, kernel_size=3, strides=
 
 @backend.register_keras_serializable(package="kecamCommon")
 class CompatibleExtractPatches(layers.Layer):
-    def __init__(self, sizes=3, strides=2, rates=1, padding="SAME", compressed=True, force_conv=False, **kwargs):
+    def __init__(self, sizes=3, strides=2, rates=1, padding="same", compressed=True, force_conv=False, **kwargs):
         super().__init__(**kwargs)
         self.sizes, self.strides, self.rates, self.padding = sizes, strides, rates, padding
         self.compressed, self.force_conv = compressed, force_conv
 
         self.kernel_size = sizes[1] if isinstance(sizes, (list, tuple)) else sizes
         self.strides = strides[1] if isinstance(strides, (list, tuple)) else strides
-        self.dilation_rate = rates[1] if isinstance(rates, (list, tuple)) else rates
+        # dilation_rate can be 2 different values, used in DiNAT
+        self.dilation_rate = (rates if len(rates) == 2 else rates[1:3]) if isinstance(rates, (list, tuple)) else (rates, rates)
         self.filters = self.kernel_size * self.kernel_size
 
         if backend.backend() == "tensorflow":
@@ -558,14 +594,15 @@ class CompatibleExtractPatches(layers.Layer):
         else:
             self.use_conv = force_conv
 
-        self.use_layer_as_module = True  # For PyTorch backend
-
     def build(self, input_shape):
         _, self.height, self.width, self.channel = input_shape
-        if self.padding.upper() == "SAME":
-            pad = self.kernel_size // 2
-            self.pad_value = [[0, 0], [pad, pad], [pad, pad], [0, 0]]
-            self.height, self.width = self.height + pad * 2, self.width + pad * 2
+        if self.padding.lower() == "same":
+            pad_value = self.kernel_size // 2
+            self.pad_value_list = [[0, 0], [pad_value, pad_value], [pad_value, pad_value], [0, 0]]
+            self.height, self.width = self.height + pad_value * 2, self.width + pad_value * 2
+            self.pad_value = pad_value
+        else:
+            self.pad_value = 0
 
         if self.use_conv:
             self.conv = layers.Conv2D(
@@ -573,7 +610,7 @@ class CompatibleExtractPatches(layers.Layer):
                 kernel_size=self.kernel_size,
                 strides=self.strides,
                 dilation_rate=self.dilation_rate,
-                padding="VALID",
+                padding="valid",
                 use_bias=False,
                 trainable=False,
                 kernel_initializer=__unfold_filters_initializer__,
@@ -583,14 +620,14 @@ class CompatibleExtractPatches(layers.Layer):
         else:
             self._sizes_ = [1, self.kernel_size, self.kernel_size, 1]
             self._strides_ = [1, self.strides, self.strides, 1]
-            self._rates_ = [1, self.dilation_rate, self.dilation_rate, 1]
+            self._rates_ = [1, *self.dilation_rate, 1]
         # output_size = backend.compute_conv_output_size([self.height, self.width], self.kernel_size, self.strides, self.padding, self.dilation_rate)
         # self.output_height, self.output_width = output_size
         super().build(input_shape)
 
     def call(self, inputs):
-        if self.padding.upper() == "SAME":
-            inputs = functional.pad(inputs, self.pad_value)
+        if self.pad_value > 0:
+            inputs = functional.pad(inputs, self.pad_value_list)
 
         if self.use_conv:
             merge_channel = functional.transpose(inputs, [0, 3, 1, 2])
@@ -605,7 +642,7 @@ class CompatibleExtractPatches(layers.Layer):
             else:
                 out = functional.reshape(out, [-1, conv_rr.shape[1], conv_rr.shape[2], self.kernel_size, self.kernel_size, self.channel])
         else:
-            out = functional.extract_patches(inputs, self._sizes_, self._strides_, self._rates_, "VALID")
+            out = functional.extract_patches(inputs, self._sizes_, self._strides_, self._rates_, "VALID")  # must be upper word VALID/SAME
             if not self.compressed:
                 # [batch, hh, ww, kernel, kernel, channnel]
                 out = functional.reshape(out, [-1, out.shape[1], out.shape[2], self.kernel_size, self.kernel_size, self.channel])
@@ -626,6 +663,34 @@ class CompatibleExtractPatches(layers.Layer):
         return base_config
 
 
+""" Preprocess input and decode predictions """
+
+
+def init_mean_std_by_rescale_mode(rescale_mode, convert_to_image_data_format=True):
+    if isinstance(rescale_mode, (list, tuple)):  # Specific mean and std
+        mean, std = rescale_mode
+    elif rescale_mode == "torch":
+        mean = np.array([0.485, 0.456, 0.406]).astype("float32") * 255.0
+        std = np.array([0.229, 0.224, 0.225]).astype("float32") * 255.0
+        if backend.image_data_format() != "channels_last" and convert_to_image_data_format:
+            mean, std = mean[:, None, None], std[:, None, None]
+    elif rescale_mode == "tf":  # [0, 255] -> [-1, 1]
+        mean, std = 127.5, 127.5
+        # mean, std = 127.5, 128.0
+    elif rescale_mode == "tf128":  # [0, 255] -> [-1, 1]
+        mean, std = 128.0, 128.0
+    elif rescale_mode == "raw01":
+        mean, std = 0, 255.0  # [0, 255] -> [0, 1]
+    elif rescale_mode == "clip":  # value from openai/CLIP
+        mean = np.array([0.48145466, 0.4578275, 0.40821073]).astype("float32") * 255.0
+        std = np.array([0.26862954, 0.26130258, 0.27577711]).astype("float32") * 255.0
+        if backend.image_data_format() != "channels_last" and convert_to_image_data_format:
+            mean, std = mean[:, None, None], std[:, None, None]
+    else:
+        mean, std = 0, 1  # raw inputs [0, 255]
+    return mean, std
+
+
 class PreprocessInput:
     """`rescale_mode` `torch` means `(image - [0.485, 0.456, 0.406]) / [0.229, 0.224, 0.225]`, `tf` means `(image - 0.5) / 0.5`"""
 
@@ -637,28 +702,15 @@ class PreprocessInput:
         input_shape = input_shape[1:] if len(input_shape) == 4 else input_shape
         if None in input_shape:
             self.input_shape = (None, None)  # Dynamic input_shape
+        elif len(input_shape) == 2:
+            self.input_shape = input_shape
         else:
             channel_axis, channel_dim = min(enumerate(input_shape), key=lambda xx: xx[1])  # Assume the smallest value is the channel dimension
             self.input_shape = [dim for axis, dim in enumerate(input_shape) if axis != channel_axis]
 
     def set_rescale_mode(self, rescale_mode):
-        if isinstance(rescale_mode, (list, tuple)):  # Specific mean and std
-            mean, std = rescale_mode
-        elif rescale_mode == "torch":
-            mean = np.array([0.485, 0.456, 0.406]).astype("float32") * 255.0
-            std = np.array([0.229, 0.224, 0.225]).astype("float32") * 255.0
-            if backend.image_data_format() != "channels_last":
-                mean, std = mean[:, None, None], std[:, None, None]
-        elif rescale_mode == "tf":  # [0, 255] -> [-1, 1]
-            mean, std = 127.5, 127.5
-            # mean, std = 127.5, 128.0
-        elif rescale_mode == "tf128":  # [0, 255] -> [-1, 1]
-            mean, std = 128.0, 128.0
-        elif rescale_mode == "raw01":
-            mean, std = 0, 255.0  # [0, 255] -> [0, 1]
-        else:
-            mean, std = 0, 1  # raw inputs [0, 255]
-        self.mean, self.std, self.rescale_mode = mean, std, rescale_mode
+        self.mean, self.std = init_mean_std_by_rescale_mode(rescale_mode)
+        self.rescale_mode = rescale_mode
 
     def __call__(self, image, resize_method="bilinear", resize_antialias=False, input_shape=None):
         if input_shape is not None:
@@ -673,10 +725,13 @@ class PreprocessInput:
         return images
 
 
-def add_pre_post_process(model, rescale_mode="tf", input_shape=None, post_process=None):
+def add_pre_post_process(model, rescale_mode="tf", input_shape=None, post_process=None, features=None):
     from keras_cv_attention_models.imagenet.eval_func import decode_predictions
 
     input_shape = model.input_shape[1:] if input_shape is None else input_shape
     model.preprocess_input = PreprocessInput(input_shape, rescale_mode=rescale_mode)
     model.decode_predictions = decode_predictions if post_process is None else post_process
     model.rescale_mode = rescale_mode
+
+    if features is not None:
+        model.extract_features = lambda: features
