@@ -2,6 +2,7 @@ import numpy as np
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models, functional, image_data_format
 from keras_cv_attention_models.aotnet import AotNet
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.download_and_load import reload_model_weights
 from keras_cv_attention_models.attention_layers import batchnorm_with_activation, conv2d_no_bias, CompatibleExtractPatches, group_norm
 
@@ -54,8 +55,11 @@ def cot_attention(inputs, kernel_size=3, strides=1, downsample_first=True, activ
     # y2 = (ww.unsqueeze(2) * x2.unsqueeze(1)).sum(-3).view(-1, filters, height, width)
     # sizes, patch_strides = [1, kernel_size, kernel_size, 1], [1, 1, 1, 1]
     # embed = layers.ZeroPadding2D(padding=kernel_size // 2, name=name and name + "embed_pad")(embed)
-    # embed = functional.extract_patches(embed, sizes=sizes, strides=patch_strides, rates=(1, 1, 1, 1), padding="VALID")
-    embed = CompatibleExtractPatches(sizes=kernel_size, strides=1, name=name and name + "patchs_")(embed)
+    # embed = functional.extract_patches(embed, sizes=sizes, strides=patch_strides, rates=(1, 1, 1, 1), padding="valid")
+    if backend.is_torch_backend:
+        embed = functional.extract_patches(embed, sizes=kernel_size, strides=1, padding="same", data_format=image_data_format())
+    else:
+        embed = CompatibleExtractPatches(sizes=kernel_size, strides=1, padding="same", name=name and name + "patchs_")(embed)
 
     if image_data_format() == "channels_last":
         embed_ww = functional.reshape(embed_ww, (-1, height, width, filters // reduction, kernel_size * kernel_size))
@@ -66,6 +70,8 @@ def cot_attention(inputs, kernel_size=3, strides=1, downsample_first=True, activ
         embed_ww = functional.reshape(embed_ww, (-1, filters // reduction, kernel_size * kernel_size, height, width))
         reduction_axis, kernel_axis = 2, 1
         embed_ww = functional.expand_dims(functional.transpose(embed_ww, [0, 2, 1, 3, 4]), axis=reduction_axis)  # expand dim on `reduction` axis
+        embed = functional.reshape(embed, [-1, filters, kernel_size * kernel_size, height * width])
+        embed = functional.transpose(embed, [0, 2, 1, 3])
         embed = functional.reshape(embed, [-1, kernel_size * kernel_size, reduction, filters // reduction, height, width])
 
     embed_out = layers.Multiply(name=name and name + "local_conv_mul")([embed, embed_ww])
@@ -105,18 +111,21 @@ def CotNet(input_shape=(224, 224, 3), bn_after_attn=False, shortcut_type="avg",
     return model
 
 
+@register_model
 def CotNet50(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [3, 4, 6, 3]
     strides = [1, 2, 2, 2]
     return CotNet(**locals(), **kwargs, model_name="cotnet50")
 
 
+@register_model
 def CotNet101(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [3, 4, 23, 3]
     strides = [1, 2, 2, 2]
     return CotNet(**locals(), **kwargs, model_name="cotnet101")
 
 
+@register_model
 def CotNetSE50D(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [3, 4, 6, 3]
     strides = [2, 2, 2, 2]
@@ -133,6 +142,7 @@ def CotNetSE50D(input_shape=(224, 224, 3), num_classes=1000, activation="relu",
     return CotNet(**locals(), **kwargs, model_name="cotnet_se50d")
 
 
+@register_model
 def CotNetSE101D(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [3, 4, 23, 3]
     strides = [2, 2, 2, 2]
@@ -149,6 +159,7 @@ def CotNetSE101D(input_shape=(224, 224, 3), num_classes=1000, activation="relu",
     return CotNet(**locals(), **kwargs, model_name="cotnet_se101d")
 
 
+@register_model
 def CotNetSE152D(input_shape=(224, 224, 3), num_classes=1000, activation="relu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [3, 8, 36, 3]
     strides = [2, 2, 2, 2]
