@@ -2,10 +2,10 @@
 Creates a EfficientNetV2 Model as defined in: Mingxing Tan, Quoc V. Le. (2021). arXiv preprint arXiv:2104.00298.
 EfficientNetV2: Smaller Models and Faster Training.
 """
-import os
-import tensorflow as tf
-from tensorflow import keras
-from tensorflow.keras import backend as K
+import numpy as np
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, functional, models, is_channels_last
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import (
     batchnorm_with_activation,
     conv2d_no_bias,
@@ -41,15 +41,18 @@ FILE_HASH_DICT = {
     "v1-b6": {"noisy_student": "20dd18b0df60cd7c0387c8af47bd96f8", "imagenet": "da13735af8209f675d7d7d03a54bfa27"},
     "v1-b7": {"noisy_student": "7f6f6dd4e8105e32432607ad28cfad0f", "imagenet": "d9c22b5b030d1e4f4c3a96dbf5f21ce6"},
     "v1-l2": {"noisy_student": "5fedc721febfca4b08b03d1f18a4a3ca"},
+    "edgetpu-large": {"imagenet": "e92d2a9f12611d10e1b8843ab43d578c"},
+    "edgetpu-medium": {"imagenet": "40a682a333496501d49f060c51e9858f"},
+    "edgetpu-small": {"imagenet": "89ec826f3682f52ec4ba73775dfb963b"},
 }
 
 
 def inverted_residual_block(
     inputs,
     output_channel,
-    stride,
-    expand,
-    shortcut,
+    stride=1,
+    expand=4,
+    shortcut=False,
     kernel_size=3,
     drop_rate=0,
     se_ratio=0,
@@ -59,11 +62,11 @@ def inverted_residual_block(
     se_divisor=1,  # 8 for mobilenetv3
     se_limit_round_down=0.9,  # 0.95 for fbnet
     use_global_context_instead_of_se=False,
+    use_last_bn_zero_gamma=False,
     activation="swish",
     name=None,
 ):
-    channel_axis = 1 if K.image_data_format() == "channels_first" else -1
-    input_channel = inputs.shape[channel_axis]
+    input_channel = inputs.shape[-1 if is_channels_last() else 1]
     bn_eps = TORCH_BATCH_NORM_EPSILON if is_torch_mode else TF_BATCH_NORM_EPSILON
     hidden_channel = make_divisible(input_channel * expand, 8)
 
@@ -77,12 +80,14 @@ def inverted_residual_block(
         nn = inputs
 
     if not is_fused:
-        if is_torch_mode and kernel_size // 2 > 0:
-            nn = keras.layers.ZeroPadding2D(padding=kernel_size // 2, name=name and name + "pad")(nn)
-            padding = "VALID"
+        if is_torch_mode and backend.is_torch_backend and kernel_size // 2 > 0:
+            padding = kernel_size // 2
+        elif is_torch_mode and kernel_size // 2 > 0:
+            nn = layers.ZeroPadding2D(padding=kernel_size // 2, name=name and name + "pad")(nn)
+            padding = "valid"
         else:
-            padding = "SAME"
-        nn = keras.layers.DepthwiseConv2D(kernel_size, padding=padding, strides=stride, use_bias=False, name=name and name + "MB_dw_")(nn)
+            padding = "same"
+        nn = layers.DepthwiseConv2D(kernel_size, padding=padding, strides=stride, use_bias=False, name=name and name + "MB_dw_")(nn)
         nn = batchnorm_with_activation(nn, activation=activation, epsilon=bn_eps, name=name and name + "MB_dw_")
 
     if se_ratio > 0:
@@ -96,16 +101,16 @@ def inverted_residual_block(
     # pw-linear
     if is_fused and expand == 1:
         nn = conv2d_no_bias(nn, output_channel, 3, strides=stride, padding="same", use_torch_padding=is_torch_mode, name=name and name + "fu_")
-        nn = batchnorm_with_activation(nn, activation=activation, epsilon=bn_eps, name=name and name + "fu_")
+        nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=use_last_bn_zero_gamma, epsilon=bn_eps, name=name and name + "fu_")
     else:
         nn = conv2d_no_bias(nn, output_channel, 1, strides=1, padding="valid", use_torch_padding=is_torch_mode, name=name and name + "MB_pw_")
-        nn = batchnorm_with_activation(nn, activation=None, epsilon=bn_eps, name=name and name + "MB_pw_")
+        nn = batchnorm_with_activation(nn, activation=None, zero_gamma=use_last_bn_zero_gamma, epsilon=bn_eps, name=name and name + "MB_pw_")
 
     if shortcut:
         nn = drop_block(nn, drop_rate, name=name and name + "drop")
-        return keras.layers.Add(name=name and name + "output")([inputs, nn])
+        return layers.Add(name=name and name + "output")([inputs, nn])
     else:
-        return keras.layers.Activation("linear", name=name and name + "output")(nn)  # Identity, Just need a name here
+        return layers.Activation("linear", name=name and name + "output")(nn)  # Identity, Just need a name here
 
 
 def EfficientNetV2(
@@ -115,6 +120,7 @@ def EfficientNetV2(
     strides=[1, 2, 2, 2, 1, 2],
     se_ratios=[0, 0, 0, 0.25, 0.25, 0.25],
     is_fused="auto",  # True if se_ratio == 0 else False
+    use_shortcuts=True,  # Force shortcut usage for each stack
     first_conv_filter=32,
     output_conv_filter=1280,
     kernel_sizes=3,
@@ -137,17 +143,20 @@ def EfficientNetV2(
     # for V2 models, "21k" pretrained are all "tf", "imagenet" pretrained "bx" models are all "torch", ["s", "m", "l", "xl"] are "tf"
     rescale_mode = "tf" if pretrained is not None and pretrained.startswith("imagenet21k") else rescale_mode
 
-    inputs = keras.layers.Input(shape=input_shape)
+    # Regard input_shape as force using original shape if len(input_shape) == 4,
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
+    input_shape = backend.align_input_shape_by_image_data_format(input_shape)
+    inputs = layers.Input(shape=input_shape)
     bn_eps = TORCH_BATCH_NORM_EPSILON if is_torch_mode else TF_BATCH_NORM_EPSILON
 
     if include_preprocessing and rescale_mode == "torch":
-        channel_axis = 1 if K.image_data_format() == "channels_first" else -1
-        Normalization = keras.layers.Normalization if hasattr(keras.layers, "Normalization") else keras.layers.experimental.preprocessing.Normalization
-        mean = tf.constant([0.485, 0.456, 0.406]) * 255.0
-        std = (tf.constant([0.229, 0.224, 0.225]) * 255.0) ** 2
+        channel_axis = 1 if backend.image_data_format() == "channels_first" else -1
+        Normalization = layers.Normalization if hasattr(layers, "Normalization") else layers.experimental.preprocessing.Normalization
+        mean = np.array([0.485, 0.456, 0.406], dtype="float32") * 255.0
+        std = (np.array([0.229, 0.224, 0.225], dtype="float32") * 255.0) ** 2
         nn = Normalization(mean=mean, variance=std, axis=channel_axis)(inputs)
     elif include_preprocessing and rescale_mode == "tf":
-        Rescaling = keras.layers.Rescaling if hasattr(keras.layers, "Rescaling") else keras.layers.experimental.preprocessing.Rescaling
+        Rescaling = layers.Rescaling if hasattr(layers, "Rescaling") else layers.experimental.preprocessing.Rescaling
         nn = Rescaling(scale=1.0 / 128.0, offset=-1)(inputs)
     else:
         nn = inputs
@@ -163,6 +172,7 @@ def EfficientNetV2(
     pre_out = stem_width
     global_block_id = 0
     total_blocks = sum(depthes)
+    se_ratios = se_ratios if isinstance(se_ratios, (list, tuple)) else ([se_ratios] * len(depthes))
     kernel_sizes = kernel_sizes if isinstance(kernel_sizes, (list, tuple)) else ([kernel_sizes] * len(depthes))
     for id, (expand, out_channel, depth, stride, se_ratio, kernel_size) in enumerate(zip(expands, out_channels, depthes, strides, se_ratios, kernel_sizes)):
         out = make_divisible(out_channel, 8)
@@ -170,10 +180,11 @@ def EfficientNetV2(
             cur_is_fused = True if se_ratio == 0 else False
         else:
             cur_is_fused = is_fused[id] if isinstance(is_fused, (list, tuple)) else is_fused
+        cur_use_shortcuts = use_shortcuts[id] if isinstance(use_shortcuts, (list, tuple)) else use_shortcuts
         for block_id in range(depth):
             name = "stack_{}_block{}_".format(id, block_id)
             stride = stride if block_id == 0 else 1
-            shortcut = True if out == pre_out and stride == 1 else False
+            shortcut = cur_use_shortcuts if out == pre_out and stride == 1 else False
             block_drop_rate = drop_connect_rate * global_block_id / total_blocks
             nn = inverted_residual_block(
                 nn, out, stride, expand, shortcut, kernel_size, block_drop_rate, se_ratio, cur_is_fused, **blocks_kwargs, activation=activation, name=name
@@ -187,13 +198,15 @@ def EfficientNetV2(
         nn = batchnorm_with_activation(nn, activation=activation, epsilon=bn_eps, name="post_")
     nn = output_block(nn, num_classes=num_classes, drop_rate=dropout, classifier_activation=classifier_activation)
 
-    model = keras.models.Model(inputs=inputs, outputs=nn, name=model_name)
+    model = models.Model(inputs=inputs, outputs=nn, name=model_name)
     add_pre_post_process(model, rescale_mode="raw" if include_preprocessing else rescale_mode)
     reload_model_weights(model, pretrained)
     return model
 
 
 def reload_model_weights(model, pretrained="imagenet"):
+    import os
+
     if pretrained is None:
         return
     if isinstance(pretrained, str) and pretrained.endswith(".h5"):
@@ -211,7 +224,7 @@ def reload_model_weights(model, pretrained="imagenet"):
         print(">>>> No pretrained available, model will be randomly initialized")
         return
 
-    if model_type.startswith("v1"):
+    if model_type.startswith("v1") or model_type.startswith("edgetpu"):
         pre_url = "https://github.com/leondgarse/keras_efficientnet_v2/releases/download/effnetv1_pretrained/efficientnet{}-{}.h5"
     else:
         pre_url = "https://github.com/leondgarse/keras_efficientnet_v2/releases/download/effnetv2_pretrained/efficientnet{}-{}.h5"
@@ -220,7 +233,7 @@ def reload_model_weights(model, pretrained="imagenet"):
     file_hash = FILE_HASH_DICT[model_type][pre_tt]
 
     try:
-        pretrained_model = keras.utils.get_file(file_name, url, cache_subdir="models/efficientnetv2", file_hash=file_hash)
+        pretrained_model = backend.get_file(file_name, url, cache_subdir="models", file_hash=file_hash)
     except:
         print("[Error] will not load weights, url not found or download failed:", url)
         return
@@ -229,6 +242,7 @@ def reload_model_weights(model, pretrained="imagenet"):
         model.load_weights(pretrained_model, by_name=True, skip_mismatch=True)
 
 
+@register_model
 def EfficientNetV2B0(input_shape=(224, 224, 3), num_classes=1000, dropout=0.2, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.0, depth 1.0
     out_channels = [16, 32, 48, 96, 112, 192]
@@ -238,6 +252,7 @@ def EfficientNetV2B0(input_shape=(224, 224, 3), num_classes=1000, dropout=0.2, c
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-b0", **kwargs)
 
 
+@register_model
 def EfficientNetV2B1(input_shape=(240, 240, 3), num_classes=1000, dropout=0.2, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.0, depth 1.1
     out_channels = [16, 32, 48, 96, 112, 192]
@@ -247,6 +262,7 @@ def EfficientNetV2B1(input_shape=(240, 240, 3), num_classes=1000, dropout=0.2, c
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-b1", **kwargs)
 
 
+@register_model
 def EfficientNetV2B2(input_shape=(260, 260, 3), num_classes=1000, dropout=0.3, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.1, depth 1.2
     out_channels = [16, 32, 56, 104, 120, 208]
@@ -256,6 +272,7 @@ def EfficientNetV2B2(input_shape=(260, 260, 3), num_classes=1000, dropout=0.3, c
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-b2", **kwargs)
 
 
+@register_model
 def EfficientNetV2B3(input_shape=(300, 300, 3), num_classes=1000, dropout=0.3, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.2, depth 1.4
     out_channels = [16, 40, 56, 112, 136, 232]
@@ -265,6 +282,7 @@ def EfficientNetV2B3(input_shape=(300, 300, 3), num_classes=1000, dropout=0.3, c
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-b3", **kwargs)
 
 
+@register_model
 def EfficientNetV2T(input_shape=(288, 288, 3), num_classes=1000, dropout=0.2, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.4 * 0.8, depth 1.8 * 0.9, from timm
     is_torch_mode = True
@@ -275,6 +293,7 @@ def EfficientNetV2T(input_shape=(288, 288, 3), num_classes=1000, dropout=0.2, cl
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-t", **kwargs)
 
 
+@register_model
 def EfficientNetV2T_GC(input_shape=(288, 288, 3), num_classes=1000, dropout=0.2, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.4 * 0.8, depth 1.8 * 0.9, from timm
     is_torch_mode = True
@@ -286,6 +305,7 @@ def EfficientNetV2T_GC(input_shape=(288, 288, 3), num_classes=1000, dropout=0.2,
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-t-gc", **kwargs)
 
 
+@register_model
 def EfficientNetV2S(input_shape=(384, 384, 3), num_classes=1000, dropout=0.2, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.4, depth 1.8
     out_channels = [24, 48, 64, 128, 160, 256]
@@ -296,6 +316,7 @@ def EfficientNetV2S(input_shape=(384, 384, 3), num_classes=1000, dropout=0.2, cl
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-s", **kwargs)
 
 
+@register_model
 def EfficientNetV2M(input_shape=(480, 480, 3), num_classes=1000, dropout=0.3, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.6, depth 2.2
     out_channels = [24, 48, 80, 160, 176, 304, 512]
@@ -309,6 +330,7 @@ def EfficientNetV2M(input_shape=(480, 480, 3), num_classes=1000, dropout=0.3, cl
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-m", **kwargs)
 
 
+@register_model
 def EfficientNetV2L(input_shape=(480, 480, 3), num_classes=1000, dropout=0.4, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     # width 1.6, depth 2.2
     out_channels = [32, 64, 96, 192, 224, 384, 640]
@@ -322,6 +344,7 @@ def EfficientNetV2L(input_shape=(480, 480, 3), num_classes=1000, dropout=0.4, cl
     return EfficientNetV2(**locals(), model_name="efficientnet_v2-l", **kwargs)
 
 
+@register_model
 def EfficientNetV2XL(input_shape=(512, 512, 3), num_classes=1000, dropout=0.4, classifier_activation="softmax", pretrained="imagenet21k-ft1k", **kwargs):
     out_channels = [32, 64, 96, 192, 256, 512, 640]
     depthes = [4, 8, 8, 16, 24, 32, 8]
