@@ -1,4 +1,4 @@
-from keras_cv_attention_models.gcvit.gcvit import GCViT, GCViT_XXTiny, GCViT_XTiny, GCViT_Tiny, GCViT_Small, GCViT_Base
+from keras_cv_attention_models.gcvit.gcvit import GCViT, GCViT_XXTiny, GCViT_XTiny, GCViT_Tiny, GCViT_Tiny2, GCViT_Small, GCViT_Small2, GCViT_Base, GCViT_Large
 
 __head_doc__ = """
 Keras implementation of [Github NVlabs/GCVit](https://github.com/NVlabs/GCVit).
@@ -18,7 +18,7 @@ __tail_doc__ = """  window_ratios: window split ratio. Each stack will calculate
   dropout: dropout rate if top layers is included.
   classifier_activation: A `str` or callable. The activation function to use on the "top" layer if `num_classes > 0`.
       Set `classifier_activation=None` to return the logits of the "top" layer.
-  pretrained: one of None or "imagenet".
+  pretrained: None or one of ["imagenet", "imagenet21k-ft1k"].
       Will try to download and load pre-trained model weights if not None.
 
 Returns:
@@ -34,13 +34,19 @@ Args:
   model_name: string, model name.
 """ + __tail_doc__ + """
 Model architectures:
-  | Model        | Params | FLOPs | Input | Top1 Acc |
-  | ------------ | ------ | ----- | ----- | -------- |
-  | GCViT_XXTiny | 12.0M  | 2.15G | 224   | 79.8     |
-  | GCViT_XTiny  | 20.0M  | 2.96G | 224   | 82.04    |
-  | GCViT_Tiny   | 28.2M  | 4.83G | 224   | 83.4     |
-  | GCViT_Small  | 51.1M  | 8.63G | 224   | 83.95    |
-  | GCViT_Base   | 90.3M  | 14.9G | 224   | 84.47    |
+  | Model           | Params | FLOPs  | Input | Top1 Acc |
+  | --------------- | ------ | ------ | ----- | -------- |
+  | GCViT_XXTiny    | 12.0M  | 2.15G  | 224   | 79.9     |
+  | GCViT_XTiny     | 20.0M  | 2.96G  | 224   | 82.0     |
+  | GCViT_Tiny      | 28.2M  | 4.83G  | 224   | 83.5     |
+  | GCViT_Tiny2     | 34.5M  | 6.28G  | 224   | 83.7     |
+  | GCViT_Small     | 51.1M  | 8.63G  | 224   | 84.3     |
+  | GCViT_Small2    | 68.6M  | 11.7G  | 224   | 84.8     |
+  | GCViT_Base      | 90.3M  | 14.9G  | 224   | 85.0     |
+  | GCViT_Large     | 202.1M | 32.8G  | 224   | 85.7     |
+  | - 21k_ft1k      | 202.1M | 32.8G  | 224   | 86.6     |
+  | - 21k_ft1k, 384 | 202.9M | 105.1G | 384   | 87.4     |
+  | - 21k_ft1k, 512 | 203.8M | 205.1G | 512   | 87.6     |
 """
 
 GCViT_XXTiny.__doc__ = __head_doc__ + """
@@ -49,5 +55,8 @@ Args:
 
 GCViT_XTiny.__doc__ = GCViT_XXTiny.__doc__
 GCViT_Tiny.__doc__ = GCViT_XXTiny.__doc__
+GCViT_Tiny2.__doc__ = GCViT_XXTiny.__doc__
 GCViT_Small.__doc__ = GCViT_XXTiny.__doc__
+GCViT_Small2.__doc__ = GCViT_XXTiny.__doc__
 GCViT_Base.__doc__ = GCViT_XXTiny.__doc__
+GCViT_Large.__doc__ = GCViT_XXTiny.__doc__
