@@ -1,23 +1,7 @@
+import os
 import tensorflow as tf
 from tensorflow import keras
-
-
-def init_mean_std_by_rescale_mode(rescale_mode):
-    if isinstance(rescale_mode, (list, tuple)):  # Specific mean and std
-        mean, std = rescale_mode
-    elif rescale_mode == "torch":
-        mean = tf.constant([0.485, 0.456, 0.406]) * 255.0
-        std = tf.constant([0.229, 0.224, 0.225]) * 255.0
-    elif rescale_mode == "tf":  # [0, 255] -> [-1, 1]
-        mean, std = 127.5, 127.5
-        # mean, std = 127.5, 128.0
-    elif rescale_mode == "tf128":  # [0, 255] -> [-1, 1]
-        mean, std = 128.0, 128.0
-    elif rescale_mode == "raw01":
-        mean, std = 0, 255.0  # [0, 255] -> [0, 1]
-    else:
-        mean, std = 0, 1  # raw inputs [0, 255]
-    return mean, std
+from keras_cv_attention_models.common_layers import init_mean_std_by_rescale_mode
 
 
 def tf_imread(file_path):
@@ -214,7 +198,7 @@ def apply_mixup_cutmix(train_dataset, mixup_alpha, cutmix_alpha, switch_prob=0.5
     return train_dataset.map(mix_func, num_parallel_calls=tf.data.AUTOTUNE)
 
 
-class RandomProcessDatapoint:
+class RandomProcessImage:
     def __init__(
         self,
         target_shape=(224, 224),
@@ -269,8 +253,8 @@ class RandomProcessDatapoint:
 
             self.token_label_align = token_label.TokenLabelAlign(num_classes=num_classes, target_num_pathes=token_label_target_patches)
 
-    def __call__(self, datapoint, token_label=None):
-        image = datapoint["image"]
+    def __call__(self, image, token_label=None):
+        # image = datapoint["image"]
         if len(image.shape) < 2:
             image = tf_imread(image)
         channel = image.shape[-1]
@@ -297,16 +281,16 @@ class RandomProcessDatapoint:
         image = tf.cast(image, tf.float32)
         image.set_shape([*self.target_shape[:2], channel])
 
-        label = datapoint["label"]
+        # label = datapoint["label"]
         if self.use_token_label and token_label is not None:
             token_label = self.token_label_align(token_label, flip_left_right, scale_hh, scale_ww, crop_hh, crop_ww)
-            return image, label, token_label
+            return image, token_label
         else:
-            return image, label
+            return image
 
 
-def evaluation_process_crop_resize(datapoint, target_shape=(224, 224), central_crop=1.0, resize_method="bilinear", antialias=False):
-    image = datapoint["image"]
+def evaluation_process_crop_resize(image, target_shape=(224, 224), central_crop=1.0, resize_method="bilinear", antialias=False):
+    # image = datapoint["image"]
     if len(image.shape) < 3:
         image = tf_imread(image)
     if central_crop > 0:  # Do not crop if central_crop == -1
@@ -316,13 +300,12 @@ def evaluation_process_crop_resize(datapoint, target_shape=(224, 224), central_c
         y, x = (height - crop_size) // 2, (width - crop_size) // 2
         image = tf.image.crop_to_bounding_box(image, y, x, crop_size, crop_size)
     image = tf.image.resize(image, target_shape, method=resize_method, antialias=antialias)
-    label = datapoint["label"]
-    return image, label
+    return image
 
 
 # Not using
-def evaluation_process_resize_crop(datapoint, target_shape=(224, 224), central_crop=1.0, resize_method="bilinear", antialias=False):
-    image = datapoint["image"]
+def evaluation_process_resize_crop(image, target_shape=(224, 224), central_crop=1.0, resize_method="bilinear", antialias=False):
+    # image = datapoint["image"]
     if len(image.shape) < 3:
         image = tf_imread(image)
     shape = tf.shape(image)
@@ -335,25 +318,71 @@ def evaluation_process_resize_crop(datapoint, target_shape=(224, 224), central_c
 
     y, x = (hh_scale - target_shape[0]) // 2, (ww_scale - target_shape[1]) // 2
     image = tf.image.crop_to_bounding_box(image, y, x, target_shape[0], target_shape[1])
-
-    label = datapoint["label"]
-    return image, label
+    return image
 
 
-def recognition_dataset_from_custom_json(data_path, with_info=False):
-    import json
+def init_from_json_or_csv_or_tsv(data_path, is_caption=False):
+    if data_path.endswith(".json"):
+        import json
 
-    with open(data_path, "r") as ff:
-        aa = json.load(ff)
+        with open(data_path, "r") as ff:
+            aa = json.load(ff)
+        test_key = "validation" if "validation" in aa else "test"
+        train, test, info = aa["train"], aa[test_key], aa.get("info", {})
+    else:
+        import csv
 
-    test_key = "validation" if "validation" in aa else "test"
-    train, test, info = aa["train"], aa[test_key], aa["info"]
-    total_images, num_classes = len(train), info["num_classes"]
-    num_channels = tf_imread(aa["train"][0]["image"]).shape[-1]
+        delimiter = "\t" if data_path.endswith(".tsv") else ","
+        label_key = "caption" if is_caption else "label"
+        train, test, info, is_train = [], [], {}, True
+        with open(data_path) as ff:
+            for ii in csv.reader(ff, delimiter=delimiter):
+                if ii[0] in ["base_path", "num_classes"]:  # special keys for info
+                    info[ii[0]] = ii[1]
+                    continue
 
-    output_signature = {"image": tf.TensorSpec(shape=(), dtype=tf.string), "label": tf.TensorSpec(shape=(), dtype=tf.int64)}
-    train_ds = tf.data.Dataset.from_generator(lambda: (ii for ii in train), output_signature=output_signature)
-    test_ds = tf.data.Dataset.from_generator(lambda: (ii for ii in test), output_signature=output_signature)
+                if ii[0] == "TEST":  # Use this as indicator for start of test set
+                    is_train = False
+                elif is_train:
+                    train.append({"image": ii[0], label_key: ii[1]})
+                else:
+                    test.append({"image": ii[0], label_key: ii[1]})
+        test_key = "test"
+
+    """ Construct more info """
+    total_images, num_classes = len(train), info.get("num_classes", 0)
+    if not is_caption and num_classes <= 0 and "label" in train[0] and isinstance(train[0]["label"], int):
+        num_classes = max([ii["label"] for ii in train]) + 1
+        print(">>>> Using max value from train as num_classes:", num_classes)
+
+    if "base_path" in info and len(info["base_path"]) > 0:
+        base_path = os.path.expanduser(info["base_path"])
+        for ii in train:
+            ii["image"] = os.path.join(base_path, ii["image"])
+        for ii in test:
+            ii["image"] = os.path.join(base_path, ii["image"])
+    num_channels = tf_imread(train[0]["image"]).shape[-1]
+    return (train, test, info, test_key), (total_images, num_classes, num_channels)
+
+
+def build_custom_dataset(data_path, with_info=False, info_only=False, caption_tokenizer=None):
+    is_caption = False if caption_tokenizer is None else True
+    (train, test, info, test_key), (total_images, num_classes, num_channels) = init_from_json_or_csv_or_tsv(data_path, is_caption)
+
+    if is_caption:
+        # from tqdm import tqdm
+        # train = [{"image": ii["image"], "caption": caption_tokenizer(ii["caption"])} for ii in tqdm(train, "Tokenizing train caption")]
+        # test = [{"image": ii["image"], "caption": caption_tokenizer(ii["caption"])} for ii in tqdm(test, "Tokenizing test caption")]
+        context_length = caption_tokenizer.context_length
+        output_signature = {"image": tf.TensorSpec(shape=(), dtype=tf.string), "caption": tf.TensorSpec(shape=(context_length,), dtype=tf.int64)}
+        train_gen = lambda: ({"image": ii["image"], "caption": caption_tokenizer(ii["caption"])} for ii in train)
+        test_gen = lambda: ({"image": ii["image"], "caption": caption_tokenizer(ii["caption"])} for ii in test)
+    else:
+        output_signature = {"image": tf.TensorSpec(shape=(), dtype=tf.string), "label": tf.TensorSpec(shape=(), dtype=tf.int64)}
+        train_gen = lambda: (ii for ii in train)
+        test_gen = lambda: (ii for ii in test)
+    train_ds = tf.data.Dataset.from_generator(train_gen, output_signature=output_signature)
+    test_ds = tf.data.Dataset.from_generator(test_gen, output_signature=output_signature)
 
     options = tf.data.Options()
     options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.DATA
@@ -425,33 +454,34 @@ def init_dataset(
     teacher_model_input_shape=-1,  # -1 means same with input_shape
     **augment_kwargs,  # Too many...
 ):
-    import tensorflow_datasets as tfds
-
     # print(">>>> Dataset args:", locals())
     is_tpu = True if len(tf.config.list_logical_devices("TPU")) > 0 else False  # Set True for try_gcs and drop_remainder
+    try_gcs, drop_remainder = is_tpu, is_tpu
     use_token_label = False if token_label_file is None else True
     use_distill = False if teacher_model is None else True
     teacher_model_input_shape = input_shape if teacher_model_input_shape == -1 else teacher_model_input_shape
 
-    if data_name.endswith(".json"):
-        dataset, total_images, num_classes, num_channels = recognition_dataset_from_custom_json(data_name, with_info=True)
+    if data_name.endswith(".json") or data_name.endswith(".tsv"):
+        if info_only:
+            _, (total_images, num_classes, num_channels) = init_from_json_or_csv_or_tsv(data_name)
+        else:
+            dataset, total_images, num_classes, num_channels = build_custom_dataset(data_name, with_info=True)
     else:
-        dataset, info = tfds.load(data_name, with_info=True, try_gcs=is_tpu)
+        import tensorflow_datasets as tfds
+
+        dataset, info = tfds.load(data_name, with_info=True, try_gcs=try_gcs)
         num_classes = info.features["label"].num_classes
         num_channels = info.features["image"].shape[-1]
         total_images = info.splits["train"].num_examples
     steps_per_epoch = int(tf.math.ceil(total_images / float(batch_size)))
-
     if info_only:
         return total_images, num_classes, steps_per_epoch, num_channels  # return num_channels in case it's not 3
 
-    """ Train dataset """
-    train_dataset = dataset["train"]
-    if use_token_label:
-        train_dataset = build_token_label_dataset(train_dataset, token_label_file)
-
+    mean, std = init_mean_std_by_rescale_mode(rescale_mode)
     AUTOTUNE = tf.data.AUTOTUNE
-    train_pre_batch = RandomProcessDatapoint(
+
+    """ Train dataset functions """
+    train_image_func = RandomProcessImage(
         target_shape=teacher_model_input_shape if use_distill else input_shape,
         central_crop=-1,  # Resize directly w/o crop, if random_crop_min not in (0, 1)
         random_crop_min=random_crop_min,
@@ -466,16 +496,22 @@ def init_dataset(
         num_classes=num_classes,
         **augment_kwargs,
     )
+
+    if use_token_label:
+        train_pre_batch = lambda data_point: (*train_image_func(data_point["image"]), data_point["label"])
+        train_post_batch = lambda xx, token_label, yy: ((xx - mean) / std, tf.one_hot(yy, num_classes), token_label)
+    else:
+        train_pre_batch = lambda data_point: (train_image_func(data_point["image"]), data_point["label"])
+        train_post_batch = lambda xx, yy: ((xx - mean) / std, tf.one_hot(yy, num_classes))
+
+    """ Train dataset """
+    train_dataset = dataset["train"]
+    if use_token_label:
+        train_dataset = build_token_label_dataset(train_dataset, token_label_file)
     if use_shuffle:
         train_dataset = train_dataset.shuffle(buffer_size, seed=seed)
-    train_dataset = train_dataset.map(train_pre_batch, num_parallel_calls=AUTOTUNE).batch(batch_size, drop_remainder=is_tpu)
-
-    mean, std = init_mean_std_by_rescale_mode(rescale_mode)
-    if use_token_label:
-        train_post_batch = lambda xx, yy, token_label: ((xx - mean) / std, tf.one_hot(yy, num_classes), token_label)
-    else:
-        train_post_batch = lambda xx, yy: ((xx - mean) / std, tf.one_hot(yy, num_classes))
-    train_dataset = train_dataset.map(train_post_batch, num_parallel_calls=AUTOTUNE)
+    train_dataset = train_dataset.map(train_pre_batch, num_parallel_calls=AUTOTUNE)
+    train_dataset = train_dataset.batch(batch_size, drop_remainder=drop_remainder).map(train_post_batch, num_parallel_calls=AUTOTUNE)
     train_dataset = apply_mixup_cutmix(train_dataset, mixup_alpha, cutmix_alpha, switch_prob=0.5)
 
     if use_token_label:
@@ -491,47 +527,62 @@ def init_dataset(
     test_dataset = dataset.get("validation", dataset.get("test", None))
     if test_dataset is not None:
         # test_pre_batch = lambda xx: evaluation_process_resize_crop(xx, input_shape[:2], eval_central_crop, resize_method, resize_antialias)  # timm
-        test_pre_batch = lambda xx: evaluation_process_crop_resize(xx, input_shape[:2], eval_central_crop, resize_method, resize_antialias)
-        test_dataset = test_dataset.map(test_pre_batch, num_parallel_calls=AUTOTUNE)
-        # Have to drop_remainder also for test set...
-        test_dataset = test_dataset.batch(batch_size, drop_remainder=is_tpu)
+        # test_image_func = lambda xx: evaluation_process_crop_resize(xx, input_shape[:2], eval_central_crop, resize_method, resize_antialias)
+        test_pre_batch = lambda data_point: (
+            evaluation_process_crop_resize(data_point["image"], input_shape[:2], eval_central_crop, resize_method, resize_antialias),
+            data_point["label"],
+        )
         if use_token_label:
             test_post_batch = lambda xx, yy: ((xx - mean) / std, (tf.one_hot(yy, num_classes), None))  # just give None on token_label data position
         elif use_distill:
             test_post_batch = lambda xx, yy: ((xx - mean) / std, (tf.one_hot(yy, num_classes), None))
         else:
             test_post_batch = lambda xx, yy: ((xx - mean) / std, tf.one_hot(yy, num_classes))
-        test_dataset = test_dataset.map(test_post_batch)
+
+        test_dataset = test_dataset.map(test_pre_batch, num_parallel_calls=AUTOTUNE)
+        # Have to drop_remainder also for test set...
+        test_dataset = test_dataset.batch(batch_size, drop_remainder=drop_remainder).map(test_post_batch)
     return train_dataset, test_dataset, total_images, num_classes, steps_per_epoch
 
 
 """ Show """
 
 
-def show_batch_sample(dataset, rescale_mode="tf", rows=-1, base_size=3):
-    from keras_cv_attention_models import visualizing
+def show_batch_sample(dataset, rescale_mode="tf", rows=-1, caption_tokenizer=None, base_size=3, indices_2_labels=None):
+    from keras_cv_attention_models import plot_func
     from keras_cv_attention_models.imagenet.eval_func import decode_predictions
 
     if isinstance(dataset, (list, tuple)):
         images, labels = dataset
-    elif isinstance(dataset.element_spec[1], tuple):
+    elif isinstance(dataset.element_spec[0], tuple):  # caption datasets
+        (images, labels), _ = dataset.as_numpy_iterator().next()
+    elif isinstance(dataset.element_spec[1], tuple):  # token_label datasets
         images, (labels, token_label) = dataset.as_numpy_iterator().next()
     else:
         images, labels = dataset.as_numpy_iterator().next()
+
+    if caption_tokenizer is not None:
+        labels = [caption_tokenizer(ii) for ii in labels]
+
     mean, std = init_mean_std_by_rescale_mode(rescale_mode)
     mean, std = (mean.numpy(), std.numpy()) if hasattr(mean, "numpy") else (mean, std)
     images = (images * std + mean) / 255
 
-    if tf.shape(labels)[-1] == 1000:
+    if isinstance(labels[0], str):
+        pass  # caption datasets
+    elif tf.shape(labels)[-1] == 1000:
         labels = [ii[0][1] for ii in decode_predictions(labels, top=1)]
     elif tf.rank(labels[0]) == 1:
         labels = tf.argmax(labels, axis=-1).numpy()  # If 2 dimension
-    ax, _ = visualizing.stack_and_plot_images(images, texts=labels, rows=rows, ax=None, base_size=base_size)
+
+    if not isinstance(labels[0], str) and indices_2_labels is not None:
+        labels = [indices_2_labels.get(label, indices_2_labels.get(str(label), str(label))) for label in labels]
+    ax, _ = plot_func.stack_and_plot_images(images, texts=labels, rows=rows, ax=None, base_size=base_size)
     return ax
 
 
 def show_token_label_patches_single(image, token_label, rescale_mode="tf", top_k=3, resize_patch_shape=(160, 160)):
-    from keras_cv_attention_models import visualizing
+    from keras_cv_attention_models import plot_func
 
     mean, std = init_mean_std_by_rescale_mode(rescale_mode)
     mean, std = (mean.numpy(), std.numpy()) if hasattr(mean, "numpy") else (mean, std)
@@ -553,4 +604,4 @@ def show_token_label_patches_single(image, token_label, rescale_mode="tf", top_k
             scores = ",".join(["{:.1f}".format(ii * 100) for ii in token_label_scores[hh_id, ww_id]])
             classes = ",".join(["{:d}".format(ii) for ii in token_label_classes[hh_id, ww_id].astype("int")])
             labels.append(classes + "\n" + scores)
-    visualizing.stack_and_plot_images(image_pathes, labels)
+    plot_func.stack_and_plot_images(image_pathes, labels)
