@@ -27,6 +27,22 @@ class TorchModelInterf:
 
 
 class TFLiteModelInterf:
+    """
+    >>> import kecam
+    >>> mm = kecam.models.AotNet50(input_shape=[None, None, 3])  # Dynamic shape
+    >>> open(mm.name + ".tflite", "wb").write(tf.lite.TFLiteConverter.from_keras_model(mm).convert())
+    >>> tt = kecam.imagenet.eval_func.TFLiteModelInterf('aotnet50.tflite')
+    >>>
+    >>> print(f"{tt.input_shape = }")
+    >>> # tt.input_shape = (1, 1, 1, 3)
+    >>> print(f"{tt([np.ones([224, 224, 3]).astype('float32')]).shape = }")
+    >>> # >>>> Calling resize_tensor_input, input_shape (1, 1, 1, 3) -> (1, 224, 224, 3):
+    >>> # tt([np.ones([224, 224, 3]).astype('float32')]).shape = (1, 1000)
+    >>> print(f"{tt([np.ones([119, 75, 3]).astype('float32')]).shape = }")
+    >>> # >>>> Calling resize_tensor_input, input_shape (1, 224, 224, 3) -> (1, 119, 75, 3):
+    >>> # tt([np.ones([119, 75, 3]).astype('float32')]).shape = (1, 1000)
+    """
+
     def __init__(self, model_path):
         import tensorflow as tf
 
@@ -37,12 +53,12 @@ class TFLiteModelInterf:
         input_details = self.interpreter.get_input_details()[0]
         self.input_dtype = input_details["dtype"]
         self.input_index = input_details["index"]
-        self.input_shape = input_details["shape"].tolist()
+        self.input_shape = tuple(input_details["shape"].tolist())
 
         output_details = self.interpreter.get_output_details()[0]
         self.output_dtype = output_details["dtype"]
         self.output_index = output_details["index"]
-        self.output_shape = output_details["shape"].tolist()
+        self.output_shape = tuple(output_details["shape"].tolist())
 
         if self.input_dtype == tf.uint8 or self.output_dtype == tf.uint8:
             self.input_scale, self.input_zero_point = input_details.get("quantization", (1.0, 0.0))
@@ -69,10 +85,18 @@ class TFLiteModelInterf:
     def __call__(self, imgs):
         # print(imgs.shape, imgs[0])
         preds = []
-        for img in imgs:
+        do_concat = True
+        for id, img in enumerate(imgs):
+            cur_input_shape = (1, *img.shape)  # For dynamic input shape. Keep batch size 1
+            if cur_input_shape != self.input_shape:
+                print(">>>> Calling resize_tensor_input, input_shape {} -> {}:".format(self.input_shape, cur_input_shape))
+                self.interpreter.resize_tensor_input(self.input_index, cur_input_shape, strict=True)
+                self.interpreter.allocate_tensors()
+                self.input_shape = cur_input_shape
+                do_concat = False if id > 0 else do_concat  # If shape changed after the first inference
             pred = self.__interf__(img)
             preds.append(pred)
-        return self.tf.concat(preds, 0).numpy()
+        return np.concatenate(preds, 0) if do_concat else preds
 
 
 class ONNXModelInterf:
@@ -82,23 +106,37 @@ class ONNXModelInterf:
 
         ort.set_default_logger_severity(3)
         model_content = onnx.load_model(model_file)
-        model_content.graph.input[0].type.tensor_type.shape.dim[0].dim_param = "None"  # Set batch_size as dynamic
-        model_content.graph.output[0].type.tensor_type.shape.dim[0].dim_param = "None"  # Set batch_size as dynamic
+        for input in model_content.graph.input:
+            input.type.tensor_type.shape.dim[0].dim_param = "None"  # Set batch_size as dynamic
+        for output in model_content.graph.output:
+            output.type.tensor_type.shape.dim[0].dim_param = "None"  # Set batch_size as dynamic
 
         self.ort_session = ort.InferenceSession(model_content.SerializeToString())
-        self.input_name = self.ort_session.get_inputs()[0].name
-        self.output_names = [self.ort_session.get_outputs()[0].name]
-        self.input_shape = self.ort_session.get_inputs()[0].shape
-        self.output_shape = self.ort_session.get_outputs()[0].shape
+        inputs, outputs = self.ort_session.get_inputs(), self.ort_session.get_outputs()
+        self.input_names, self.output_names = [ii.name for ii in inputs], [ii.name for ii in outputs]
+        self.input_name, self.output_name = self.input_names[0], self.output_names[0]
 
-        # Regard channel shape as the smallest value, and if it's on dimension 1, regard as channel first NCHW format
-        self.data_format = "NCHW" if min(self.input_shape[1:]) == self.input_shape[1] else "NHWC"
+        self.input_shape = inputs[0].shape if len(inputs) == 1 else [ii.shape for ii in inputs]
+        self.output_shape = outputs[0].shape if len(outputs) == 1 else [ii.shape for ii in outputs]
 
-    def __call__(self, imgs):
+        if len(inputs[0].shape) == 4:
+            # Regard channel shape as the smallest value, and if it's on dimension 1, regard as channel first NCHW format
+            self.data_format = "NCHW" if min(inputs[0].shape[1:]) == inputs[0].shape[1] else "NHWC"
+            self.input_channels = inputs[0].shape[1] if self.data_format == "NCHW" else inputs[0].shape[-1]
+        else:
+            self.data_format = "ND"
+            self.input_channels = None
+
+    def __call__(self, imgs, *args, **kwargs):
         imgs = imgs.numpy() if hasattr(imgs, "numpy") else imgs
-        if self.data_format == "NCHW":
+        if self.data_format == "NCHW" and imgs.shape[1] != self.input_channels:
             imgs = imgs.transpose(0, 3, 1, 2)
-        return self.ort_session.run(self.output_names, {self.input_name: imgs.astype("float32")})[0]
+
+        inputs = {name: ii for name, ii in zip(self.input_names, [imgs, *args])}
+        inputs.update(kwargs)
+        # print({kk: vv.shape for kk, vv in inputs.items()})
+        outputs = self.ort_session.run(self.output_names, inputs)
+        return outputs if len(self.output_names) > 1 else outputs[0]
         # return np.array([self.ort_session.run(self.output_names, {self.input_name: imgs[None]})[0][0] for img in imgs])
 
 
@@ -166,7 +204,7 @@ def decode_predictions(preds, top=5):
     from keras_cv_attention_models.backend import get_file
 
     preds = np.array(preds.detach() if hasattr(preds, "detach") else preds)
-    if len(preds.shape) != 2 or (preds.shape[-1] != 1000 and preds.shape[-1] != 21843):
+    if len(preds.shape) != 2 or (preds.shape[-1] not in [1000, 21843, 21841]):
         print("[Error] not imagenet or imagenet21k prediction, not supported")
         return
 
@@ -174,13 +212,17 @@ def decode_predictions(preds, top=5):
 
     global CLASS_INDEX_IMAGENET
     global CLASS_INDEX_IMAGENET21K
-    if (is_imagenet and CLASS_INDEX_IMAGENET is None) or CLASS_INDEX_IMAGENET21K is None:
+    if (is_imagenet and CLASS_INDEX_IMAGENET is None) or (not is_imagenet and CLASS_INDEX_IMAGENET21K is None):
         if is_imagenet:
-            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/efficientdet/imagenet_class_index.json"
+            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/assets/imagenet_class_index.json"
+            file_hash = "c2c37ea517e94d9795004a39431a14cb"
         else:
-            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/efficientdet/imagenet21k_class_index.json"
+            url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/assets/imagenet21k_class_index.json"
+            file_hash = "a07173727548feaea3cc855ed6341a4f"
 
-        class_index_path = get_file(origin=url)
+        class_index_path = os.path.join(os.path.expanduser("~/.keras/datasets"), os.path.basename(url))
+        print(">>>> Trying to load index file:", class_index_path)
+        class_index_path = get_file(origin=url, file_hash=file_hash)
         with open(class_index_path) as ff:
             if is_imagenet:
                 CLASS_INDEX_IMAGENET = json.load(ff)
@@ -254,106 +296,3 @@ def combine_hist_into_one(hist_list, save_file=None):
         with open(save_file, "w") as ff:
             json.dump(hh, ff)
     return hh
-
-
-def curve_fit(source, target_len=10, skip=5, use_recent=40):
-    from scipy.optimize import curve_fit
-
-    def func_curv(x, a, b, c, d):
-        pp = np.log(x)
-        # pp = 1 / x
-        return a * pp**3 + b * pp**2 + c * pp + d
-
-    recent_source = source[skip:]
-    use_recent = len(source) if use_recent == -1 else use_recent
-    if len(recent_source) > use_recent:
-        recent_source = recent_source[-use_recent:]
-    start_pos = len(source) - len(recent_source)
-    popt, pcov = curve_fit(func_curv, np.arange(start_pos, len(source)), recent_source)
-    return list(source[: -len(recent_source)]) + func_curv(np.arange(start_pos, len(source) + target_len), *popt).tolist()
-
-
-def plot_and_peak_scatter(ax, source_array, peak_method, label, skip_first=0, color=None, va="bottom", pred_curve=0, **kwargs):
-    array = source_array[skip_first:]
-    for id, ii in enumerate(array):
-        if np.isnan(ii):
-            array[id] = array[id - 1]
-    ax.plot(range(skip_first, skip_first + len(array)), array, label=label, color=color, **kwargs)
-    color = ax.lines[-1].get_color() if color is None else color
-    pp = peak_method(array)
-    vv = array[pp]
-    ax.scatter(pp + skip_first, vv, color=color, marker="v")
-    # ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, fontsize=fontsize, rotation=0)
-    ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, rotation=0)
-
-    if pred_curve > 0:
-        kwargs.pop("linestyle", None)
-        pred_array = curve_fit(source_array, pred_curve)[skip_first:]
-        ax.plot(range(skip_first, skip_first + len(pred_array)), pred_array, color=color, linestyle=":", **kwargs)
-
-
-def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["bottom"], skip_first=0, pred_curve=0):
-    import matplotlib as mpl
-    import matplotlib.pyplot as plt
-
-    mpl.rcParams["lines.linewidth"] = base_size / 8 * 1.5  # 8 -> 1.5
-    mpl.rcParams["font.size"] = base_size * 2 - 4  # 8 -> 12, 6 -> 8
-    mpl.rcParams["legend.fontsize"] = base_size * 2 - 4
-    mpl.rcParams["xtick.labelsize"] = base_size * 2 - 4
-    mpl.rcParams["ytick.labelsize"] = base_size * 2 - 4
-    mpl.rcParams["xtick.major.pad"] = base_size / 2 - 2  # 8 -> 4, 6 -> 3
-    mpl.rcParams["ytick.major.pad"] = base_size / 2 - 2
-
-    num_axes = (2 + len(addition_plots)) if addition_plots is not None and len(addition_plots) != 0 else 2
-    fig, axes = plt.subplots(1, num_axes, figsize=(num_axes * base_size, base_size))
-    hists = [hists] if isinstance(hists, (str, dict)) else hists
-    names = names if isinstance(names, (list, tuple)) else [names]
-    for id, hist in enumerate(hists):
-        name = names[min(id, len(names) - 1)] if names != None else None
-        cur_va = text_va[id % len(text_va)]
-        if isinstance(hist, str):
-            name = name if name != None else os.path.splitext(os.path.basename(hist))[0]
-            with open(hist, "r") as ff:
-                hist = json.load(ff)
-        name = name if name != None else str(id)
-
-        acc_key = "acc"
-        if acc_key not in hist:
-            all_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" not in ii]
-            acc_key = "acc" if len(all_acc_key) == 0 else all_acc_key[0]
-
-        val_acc_key = "val_acc"
-        if "val_ap_ar" in hist:  # It's coco hist
-            hist["val_ap_0.50:0.95"] = [ii[0] for ii in hist["val_ap_ar"]]
-            val_acc_key = "val_ap 0.50:0.95"
-        elif val_acc_key not in hist:
-            all_val_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" in ii]
-            val_acc_key = "val_acc" if len(all_val_acc_key) == 0 else all_val_acc_key[0]
-
-        cur_pred_curve = pred_curve[min(id, len(pred_curve) - 1)] if isinstance(pred_curve, (list, tuple)) else pred_curve
-        plot_and_peak_scatter(axes[0], hist["loss"], np.argmin, name + " loss", skip_first, None, cur_va, pred_curve=cur_pred_curve)
-        color = axes[0].lines[-1].get_color()
-        val_loss = hist.get("val_loss", [])
-        if len(val_loss) > 0 and "val_loss" not in addition_plots:
-            plot_and_peak_scatter(axes[0], val_loss, np.argmin, name + " val_loss", skip_first, color, cur_va, cur_pred_curve, linestyle="--")
-
-        acc = hist.get(acc_key, [])
-        if len(acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " " + acc_key, skip_first, color, cur_va, cur_pred_curve)
-
-        val_acc = hist.get(val_acc_key, [])
-        if len(val_acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " " + val_acc_key, skip_first, color, cur_va, cur_pred_curve, linestyle="--")
-
-        if addition_plots is not None and len(addition_plots) != 0:
-            for id, ii in enumerate(addition_plots):
-                if len(hist.get(ii, [])) > 0:
-                    peak_method = np.argmin if "loss" in ii else np.argmax
-                    plot_and_peak_scatter(axes[2 + id], hist[ii], peak_method, name + " " + ii, skip_first, color, cur_va, cur_pred_curve)
-    for ax in axes:
-        ax.legend()
-        ax.grid(True)
-    fig.tight_layout()
-    fig.tight_layout()  # again
-    plt.show()
-    return fig
