@@ -1,7 +1,8 @@
 import math
 import numpy as np
 from keras_cv_attention_models import backend
-from keras_cv_attention_models.backend import layers, models, functional, image_data_format, initializers, register_keras_serializable
+from keras_cv_attention_models.backend import layers, models, functional, image_data_format, initializers
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.download_and_load import reload_model_weights
 from keras_cv_attention_models.attention_layers import (
     activation_by_name,
@@ -21,15 +22,21 @@ PRETRAINED_DICT = {
 }
 
 
-@register_keras_serializable(package="levit")
+@backend.register_keras_serializable(package="levit")
 class MultiHeadPositionalEmbedding(layers.Layer):
     def __init__(self, query_height=-1, key_height=-1, **kwargs):
         super(MultiHeadPositionalEmbedding, self).__init__(**kwargs)
         self.query_height, self.key_height = query_height, key_height
 
     def build(self, input_shape, **kwargs):
-        _, num_heads, qq_blocks, kk_blocks = input_shape
-        self.bb = self.add_weight(name="positional_embedding", shape=(kk_blocks, num_heads), initializer="zeros", trainable=True)
+        if len(input_shape) == 3:
+            _, qq_blocks, kk_blocks = input_shape
+            self.bb = self.add_weight(name="positional_embedding", shape=(kk_blocks,), initializer="zeros", trainable=True)
+            self.output_perm = None
+        else:
+            _, num_heads, qq_blocks, kk_blocks = input_shape
+            self.bb = self.add_weight(name="positional_embedding", shape=(kk_blocks, num_heads), initializer="zeros", trainable=True)
+            self.output_perm = [2, 0, 1]
 
         if self.query_height == -1:
             q_blocks_h = q_blocks_w = int(float(qq_blocks) ** 0.5)  # hh == ww
@@ -64,8 +71,8 @@ class MultiHeadPositionalEmbedding(layers.Layer):
         super(MultiHeadPositionalEmbedding, self).build(input_shape)
 
     def call(self, inputs, **kwargs):
-        pos_bias = functional.transpose(functional.gather(self.bb, self.bb_pos), [2, 0, 1])
-        return inputs + pos_bias
+        pos_bias = functional.gather(self.bb, self.bb_pos)
+        return inputs + (pos_bias if self.output_perm is None else functional.transpose(pos_bias, self.output_perm))
 
     def get_config(self):
         base_config = super().get_config()
@@ -79,7 +86,12 @@ class MultiHeadPositionalEmbedding(layers.Layer):
             source_bb = source_layer.bb  # layer
         source_bb = np.array(source_bb.detach() if hasattr(source_bb, "detach") else source_bb).astype("float32")
         hh = ww = int(float(source_bb.shape[0]) ** 0.5)
-        ss = source_bb.reshape((hh, ww, source_bb.shape[-1]))  # [hh, ww, num_heads]
+        if hh == self.k_blocks_h and ww == self.k_blocks_w:
+            self.set_weights([source_bb])
+            return
+
+        num_heads = source_bb.shape[-1] if len(source_bb.shape) == 2 else 1
+        ss = source_bb.reshape((hh, ww, num_heads))  # [hh, ww, num_heads]
         # target_hh = target_ww = int(float(self.bb.shape[0]) ** 0.5)
         tt = backend.numpy_image_resize(ss, target_shape=[self.k_blocks_h, self.k_blocks_w], method=method)  # [target_hh, target_ww, num_heads]
         tt = tt.reshape((self.bb.shape))  # [target_hh * target_ww, num_heads]
@@ -101,22 +113,28 @@ class MultiHeadPositionalEmbedding(layers.Layer):
 
 
 def mhsa_with_multi_head_position(
-    inputs, num_heads, key_dim=-1, output_dim=-1, attn_ratio=1, use_bn=False, qkv_bias=False, out_bias=False, activation=None, name=None
+    inputs, num_heads, key_dim=-1, attn_height=-1, output_dim=-1, attn_ratio=1, use_bn=False, qkv_bias=False, out_bias=False, activation=None, name=None
 ):
-    _, height, width, input_channels = inputs.shape
+    input_channels = inputs.shape[-1]
+    input_blocks = inputs.shape[1:-1]
     key_dim = key_dim if key_dim > 0 else input_channels // num_heads
     output_dim = output_dim if output_dim > 0 else input_channels
     embed_dim = key_dim * num_heads
+    if len(inputs.shape) == 4:
+        attn_height, attn_width = inputs.shape[1:-1]
+    else:
+        attn_height = attn_height if attn_height > 0 else int(inputs.shape[1] ** 0.5)  # inputs is square shape, attn_height == attn_width
+        attn_width = inputs.shape[1] // attn_height
 
     qkv_dim = (attn_ratio + 1 + 1) * embed_dim
     qkv = layers.Dense(qkv_dim, use_bias=qkv_bias, name=name and name + "qkv")(inputs)
     qkv = batchnorm_with_activation(qkv, activation=None, axis=-1, name=name and name + "qkv_") if use_bn else qkv
-    qkv = functional.reshape(qkv, (-1, qkv.shape[1] * qkv.shape[2], num_heads, qkv_dim // num_heads))
+    qkv = functional.reshape(qkv, (-1, int(np.prod(qkv.shape[1:-1])), num_heads, qkv_dim // num_heads))
     qq, kk, vv = functional.split(qkv, [key_dim, key_dim, key_dim * attn_ratio], axis=-1)
     qq, kk, vv = functional.transpose(qq, [0, 2, 1, 3]), functional.transpose(kk, [0, 2, 3, 1]), functional.transpose(vv, [0, 2, 1, 3])
 
-    output_shape = (height, width, output_dim)
-    pos_emb = MultiHeadPositionalEmbedding(query_height=height, name=name and name + "attn_pos")
+    output_shape = (*input_blocks, output_dim)
+    pos_emb = MultiHeadPositionalEmbedding(query_height=attn_height, name=name and name + "attn_pos")
     output = scaled_dot_product_attention(qq, kk, vv, output_shape, pos_emb=pos_emb, out_weight=False, name=name)
 
     if activation:
@@ -166,7 +184,7 @@ def mhsa_with_multi_head_position_and_strides(
 
 
 def res_mhsa_with_multi_head_position(inputs, embed_dim, num_heads, key_dim, attn_ratio, drop_rate=0, activation="hard_swish", name=""):
-    nn = mhsa_with_multi_head_position(inputs, num_heads, key_dim, embed_dim, attn_ratio, use_bn=True, activation=activation, name=name)
+    nn = mhsa_with_multi_head_position(inputs, num_heads, key_dim, output_dim=embed_dim, attn_ratio=attn_ratio, use_bn=True, activation=activation, name=name)
     if drop_rate > 0:
         nn = layers.Dropout(drop_rate, noise_shape=(None, 1, 1), name=name + "drop")(nn)
     return layers.Add(name=name + "add")([inputs, nn])
@@ -230,13 +248,13 @@ def LeViT(
     drop_connect_rate=0,
     dropout=0,
     classifier_activation=None,
-    use_distillation=True,
+    use_distillation=False,
     pretrained="imagenet",
     model_name="levit",
     kwargs=None,
 ):
     # Regard input_shape as force using original shape if len(input_shape) == 4,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
     inputs = layers.Input(input_shape)
     nn = patch_stem(inputs, patch_channel, activation=activation, name="stem_")
@@ -273,22 +291,34 @@ def LeViT(
             out = [out, distill]
 
     model = models.Model(inputs, out, name=model_name)
-    add_pre_post_process(model, rescale_mode="torch")
     reload_model_weights(model, PRETRAINED_DICT, "levit", pretrained, MultiHeadPositionalEmbedding)
+    add_pre_post_process(model, rescale_mode="torch")
+    model.switch_to_deploy = lambda: switch_to_deploy(model)
     return model
 
 
-def LeViT128S(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True, classifier_activation=None, pretrained="imagenet", **kwargs):
+def switch_to_deploy(model):
+    from keras_cv_attention_models.model_surgery.model_surgery import fuse_distill_head
+
+    new_model = fuse_distill_head(model) if "head" in model.output_names else model
+    add_pre_post_process(new_model, rescale_mode=model.preprocess_input.rescale_mode, post_process=model.decode_predictions)
+    return new_model
+
+
+@register_model
+def LeViT128S(input_shape=(224, 224, 3), num_classes=1000, use_distillation=False, classifier_activation=None, pretrained="imagenet", **kwargs):
     return LeViT(**locals(), model_name="levit128s", **kwargs)
 
 
-def LeViT128(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True, classifier_activation=None, pretrained="imagenet", **kwargs):
+@register_model
+def LeViT128(input_shape=(224, 224, 3), num_classes=1000, use_distillation=False, classifier_activation=None, pretrained="imagenet", **kwargs):
     num_heads = [4, 8, 12]
     depthes = [4, 4, 4]
     return LeViT(**locals(), model_name="levit128", **kwargs)
 
 
-def LeViT192(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True, classifier_activation=None, pretrained="imagenet", **kwargs):
+@register_model
+def LeViT192(input_shape=(224, 224, 3), num_classes=1000, use_distillation=False, classifier_activation=None, pretrained="imagenet", **kwargs):
     patch_channel = 192
     out_channels = [288, 384, 384]
     num_heads = [3, 5, 6]
@@ -297,6 +327,7 @@ def LeViT192(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True,
     return LeViT(**locals(), model_name="levit192", **kwargs)
 
 
+@register_model
 def LeViT256(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True, classifier_activation=None, pretrained="imagenet", **kwargs):
     patch_channel = 256
     out_channels = [384, 512, 512]
@@ -306,6 +337,7 @@ def LeViT256(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True,
     return LeViT(**locals(), model_name="levit256", **kwargs)
 
 
+@register_model
 def LeViT384(input_shape=(224, 224, 3), num_classes=1000, use_distillation=True, classifier_activation=None, pretrained="imagenet", **kwargs):
     patch_channel = 384
     out_channels = [512, 768, 768]
