@@ -1,6 +1,7 @@
 import numpy as np
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models, functional, initializers
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import activation_by_name, add_pre_post_process
 from keras_cv_attention_models.download_and_load import reload_model_weights
 
@@ -17,7 +18,10 @@ class ChannelAffine(layers.Layer):
     def __init__(self, use_bias=True, weight_init_value=1, axis=-1, **kwargs):
         super().__init__(**kwargs)
         self.use_bias, self.weight_init_value, self.axis = use_bias, weight_init_value, axis
-        self.ww_init = initializers.Constant(weight_init_value) if weight_init_value != 1 else "ones"
+        if isinstance(weight_init_value, (int, float)):
+            self.ww_init = initializers.Constant(weight_init_value) if weight_init_value != 1 else "ones"
+        else:
+            self.ww_init = weight_init_value  # Regard as built initializer
         self.bb_init = "zeros"
         self.supports_masking = False
 
@@ -57,7 +61,7 @@ class ChannelAffine(layers.Layer):
 
     def get_config(self):
         config = super().get_config()
-        config.update({"use_bias": self.use_bias, "weight_init_value": self.weight_init_value, "axis": self.axis})
+        config.update({"use_bias": self.use_bias, "axis": self.axis})
         return config
 
 
@@ -113,7 +117,7 @@ def ResMLP(
     kwargs=None,
 ):
     # Regard input_shape as force using original shape if len(input_shape) == 4,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
     inputs = layers.Input(input_shape)
     nn = layers.Conv2D(stem_width, kernel_size=patch_size, strides=patch_size, padding="valid", name="stem")(inputs)
@@ -173,17 +177,21 @@ BLOCK_CONFIGS = {
 }
 
 
+@register_model
 def ResMLP12(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     return ResMLP(**BLOCK_CONFIGS["12"], **locals(), model_name="resmlp12", **kwargs)
 
 
+@register_model
 def ResMLP24(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     return ResMLP(**BLOCK_CONFIGS["24"], **locals(), model_name="resmlp24", **kwargs)
 
 
+@register_model
 def ResMLP36(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     return ResMLP(**BLOCK_CONFIGS["36"], **locals(), model_name="resmlp36", **kwargs)
 
 
+@register_model
 def ResMLP_B24(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     return ResMLP(**BLOCK_CONFIGS["b24"], **locals(), model_name="resmlp_b24", **kwargs)
