@@ -1,6 +1,7 @@
 import math
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models, functional, image_data_format
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import (
     activation_by_name,
     add_with_layer_scale_and_drop_block,
@@ -49,7 +50,7 @@ def bottle_in_linear_out_block(inputs, out_channel, strides=1, expand_ratio=4, u
     hidden_dim = int(input_channel * expand_ratio)
     deep = conv2d_no_bias(inputs, hidden_dim, kernel_size=1, strides=1, name=name + "deep_1_")
     deep = batchnorm_with_activation(deep, activation=activation, name=name + "deep_1_")
-    deep = depthwise_conv2d_no_bias(deep, kernel_size=3, strides=strides, padding="SAME", name=name + "deep_2_")
+    deep = depthwise_conv2d_no_bias(deep, kernel_size=3, strides=strides, padding="same", name=name + "deep_2_")
     deep = batchnorm_with_activation(deep, activation=activation, name=name + "deep_2_")
     deep = conv2d_no_bias(deep, out_channel, kernel_size=1, strides=1, name=name + "deep_3_")
     deep = batchnorm_with_activation(deep, activation=None, name=name + "deep_3_")
@@ -136,9 +137,9 @@ def transformer_pre_process(inputs, out_channel, patch_size=2, resize_first=Fals
             nn = functional.resize(nn, [patch_hh * patch_size, patch_ww * patch_size], method="bilinear")
 
     if use_depthwise:  # V2
-        nn = depthwise_conv2d_no_bias(nn, kernel_size=3, strides=1, padding="SAME", name=name + "pre_1_")
+        nn = depthwise_conv2d_no_bias(nn, kernel_size=3, strides=1, padding="same", name=name + "pre_1_")
     else:  # V1
-        nn = conv2d_no_bias(nn, nn.shape[channel_axis], kernel_size=3, strides=1, padding="SAME", name=name + "pre_1_")
+        nn = conv2d_no_bias(nn, nn.shape[channel_axis], kernel_size=3, strides=1, padding="same", name=name + "pre_1_")
     nn = batchnorm_with_activation(nn, activation=activation, name=name + "pre_1_")
     nn = conv2d_no_bias(nn, out_channel, kernel_size=1, strides=1, name=name + "pre_2_")
 
@@ -202,7 +203,7 @@ def transformer_post_process(inputs, pre_attn, out_channel, patch_size=2, patch_
     nn = batchnorm_with_activation(nn, activation=activation, name=name + "post_1_")
     if pre_attn is not None:  # V1
         nn = functional.concat([pre_attn, nn], axis=channel_axis)
-        nn = conv2d_no_bias(nn, out_channel, kernel_size=3, strides=1, padding="SAME", name=name + "post_2_")
+        nn = conv2d_no_bias(nn, out_channel, kernel_size=3, strides=1, padding="same", name=name + "post_2_")
         nn = batchnorm_with_activation(nn, activation=activation, name=name + "post_2_")
     return layers.Activation("linear", name=name + "output")(nn)  # Identity, Just need a name here
 
@@ -233,13 +234,13 @@ def MobileViT(
     model_name="mobilevit",
     kwargs=None,
 ):
-    # Regard input_shape as force using original shape if first element is None or -1,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # Regard input_shape as force using original shape if len(input_shape) == 4,
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
     inputs = layers.Input(input_shape)
     nn = conv2d_no_bias(inputs, stem_width, kernel_size=3, strides=2, padding="same", name="stem_")
     nn = batchnorm_with_activation(nn, activation=activation, name="stem_")
-    height_axis, width_axis, channel_axis = (1, 2, 3) if image_data_format() == "channels_last" else (2, 3, 1)
+    height_axis, width_axis, channel_axis = (1, 2, -1) if image_data_format() == "channels_last" else (2, 3, 1)
 
     # Save line width
     mhsa_mlp_block_common_kwargs = {
@@ -276,7 +277,14 @@ def MobileViT(
                 if use_linear_attention:  # channels_last for Tensorflow, channels_first for PyTorch
                     nn = linear_mhsa_mlp_block(nn, attn_channel, layer_scale=layer_scale, **mhsa_mlp_block_common_kwargs, name=name)
                 else:  # channels_last for both Tensorflow or PyTorch
+                    if block_id == 1:
+                        block_height, block_width = nn.shape[1:-1]
+                        channel_axis = -1
+                        nn = functional.reshape(nn, [-1, block_height * block_width, nn.shape[-1]])  # Using 3D for attention inputs
                     nn = mhsa_mlp_block(nn, attn_channel, layer_scale=layer_scale, **mhsa_mlp_block_common_kwargs, name=name)
+                    if block_id == num_block - 1:
+                        channel_axis = -1 if image_data_format() == "channels_last" else 1
+                        nn = functional.reshape(nn, [-1, block_height, block_width, nn.shape[-1]])  # Revert 3D to 4D
 
                 if block_id == num_block - 1:  # post
                     norm_axis = "auto" if use_linear_attention else -1
@@ -293,6 +301,7 @@ def MobileViT(
     return model
 
 
+@register_model
 def MobileViT_XXS(input_shape=(256, 256, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [1, 3, 3, 5, 4]
     out_channels = [16, 24, 48, 64, 80]
@@ -302,6 +311,7 @@ def MobileViT_XXS(input_shape=(256, 256, 3), num_classes=1000, activation="swish
     return MobileViT(**locals(), model_name="mobilevit_xxs", **kwargs)
 
 
+@register_model
 def MobileViT_XS(input_shape=(256, 256, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [1, 3, 3, 5, 4]
     out_channels = [32, 48, 64, 80, 96]
@@ -310,6 +320,7 @@ def MobileViT_XS(input_shape=(256, 256, 3), num_classes=1000, activation="swish"
     return MobileViT(**locals(), model_name="mobilevit_xs", **kwargs)
 
 
+@register_model
 def MobileViT_S(input_shape=(256, 256, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [1, 3, 3, 5, 4]
     out_channels = [32, 64, 96, 128, 160]
