@@ -1,5 +1,6 @@
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, models
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.download_and_load import reload_model_weights
 from keras_cv_attention_models.attention_layers import batchnorm_with_activation, conv2d_no_bias, drop_block, quad_stem, add_pre_post_process
 
@@ -19,19 +20,19 @@ def quad_block(inputs, filters, groups_div=32, strides=1, conv_shortcut=False, e
         shortcut = inputs
 
     if groups != 1:  # Edge block
-        nn = conv2d_no_bias(inputs, filters, 1, strides=1, padding="VALID", name=name + "1_")
+        nn = conv2d_no_bias(inputs, filters, 1, strides=1, padding="valid", name=name + "1_")
         nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name + "1_")
     else:
         nn = inputs
 
-    nn = conv2d_no_bias(nn, filters, 3, strides=strides, padding="SAME", groups=groups, name=name + "groups_")
+    nn = conv2d_no_bias(nn, filters, 3, strides=strides, padding="same", groups=groups, name=name + "groups_")
     nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name + "2_")
 
     if extra_conv:
-        nn = conv2d_no_bias(nn, filters, 3, strides=1, padding="SAME", groups=groups, name=name + "extra_groups_")
+        nn = conv2d_no_bias(nn, filters, 3, strides=1, padding="same", groups=groups, name=name + "extra_groups_")
         nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name + "extra_2_")
 
-    nn = conv2d_no_bias(nn, expanded_filter, 1, strides=1, padding="VALID", name=name + "3_")
+    nn = conv2d_no_bias(nn, expanded_filter, 1, strides=1, padding="valid", name=name + "3_")
     nn = batchnorm_with_activation(nn, activation=None, zero_gamma=True, name=name + "3_")
 
     # print(">>>> shortcut:", shortcut.shape, "nn:", nn.shape)
@@ -73,7 +74,7 @@ def ResNetQ(
     kwargs=None,
 ):
     # Regard input_shape as force using original shape if len(input_shape) == 4,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
     inputs = layers.Input(shape=input_shape)
     nn = quad_stem(inputs, stem_width, activation=activation, stem_act=stem_act, name="stem_")
@@ -111,6 +112,7 @@ def ResNetQ(
     return model
 
 
+@register_model
 def ResNet51Q(input_shape=(224, 224, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 4, 6, 4]
     out_channels = [64, 128, 384, 384 * 4]
@@ -123,6 +125,7 @@ def ResNet51Q(input_shape=(224, 224, 3), num_classes=1000, activation="swish", c
     return ResNetQ(**locals(), model_name="resnet51q", **kwargs)
 
 
+@register_model
 def ResNet61Q(input_shape=(224, 224, 3), num_classes=1000, activation="swish", classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [1, 4, 6, 4]
     out_channels = [256, 128, 384, 384 * 4]
