@@ -9,8 +9,8 @@ from tensorflow import (
     concat,
     convert_to_tensor,
     expand_dims,
-    eye,
     gather,
+    gather_nd,
     linspace,
     map_fn,
     matmul,
@@ -29,6 +29,7 @@ from tensorflow import (
     tile,
     transpose,
     unstack,
+    where,
     zeros,
 )
 from tensorflow.image import resize, extract_patches, non_max_suppression_with_scores
