@@ -1,5 +1,6 @@
 from keras_cv_attention_models import backend
 from keras_cv_attention_models.backend import layers, functional, models, initializers, image_data_format
+from keras_cv_attention_models.models import register_model
 from keras_cv_attention_models.attention_layers import (
     BiasLayer,
     ChannelAffine,
@@ -31,10 +32,10 @@ BATCH_NORM_MOMENTUM = 0.97
 def conv_dw_pw_block(inputs, filters, kernel_size=1, strides=1, use_depthwise_conv=False, activation="swish", name=""):
     nn = inputs
     if use_depthwise_conv:
-        nn = depthwise_conv2d_no_bias(nn, kernel_size, strides, padding="SAME", name=name)
+        nn = depthwise_conv2d_no_bias(nn, kernel_size, strides, padding="same", name=name)
         nn = batchnorm_with_activation(nn, activation=activation, epsilon=BATCH_NORM_EPSILON, momentum=BATCH_NORM_MOMENTUM, name=name + "dw_")
         kernel_size, strides = 1, 1
-    nn = conv2d_no_bias(nn, filters, kernel_size, strides, padding="SAME", name=name)
+    nn = conv2d_no_bias(nn, filters, kernel_size, strides, padding="same", name=name)
     nn = batchnorm_with_activation(nn, activation=activation, epsilon=BATCH_NORM_EPSILON, momentum=BATCH_NORM_MOMENTUM, name=name)
     return nn
 
@@ -88,7 +89,7 @@ def res_spatial_pyramid_pooling(inputs, depth, expansion=0.5, pool_sizes=(5, 9,
     deep = conv_dw_pw_block(inputs, hidden_channels, kernel_size=1, activation=activation, name=name + "pre_1_")
     deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=3, activation=activation, name=name + "pre_2_")
     deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=1, activation=activation, name=name + "pre_3_")
-    pp = [layers.MaxPool2D(pool_size=ii, strides=1, padding="SAME")(deep) for ii in pool_sizes]
+    pp = [layers.MaxPool2D(pool_size=ii, strides=1, padding="same")(deep) for ii in pool_sizes]
     deep = functional.concat([deep, *pp][::-1], axis=channel_axis)  # yolor_csp.cfg, SSP concat layers=-1,-3,-5,-6
     for id in range(depth - 1):  # First one is `pre`
         deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=1, activation=activation, name=name + "post_{}_".format(id * 2 + 1))
@@ -123,7 +124,7 @@ def focus_stem(inputs, filters, kernel_size=3, strides=1, padding="valid", activ
 def csp_conv_downsample(inputs, filters, strides=2, activation="swish", name=""):
     # DownC: https://github.com/WongKinYiu/yolor/blob/paper/models/common.py#L584
     channel_axis = -1 if image_data_format() == "channels_last" else 1
-    max_down = layers.MaxPool2D(strides, strides=strides, padding="SAME")(inputs)
+    max_down = layers.MaxPool2D(strides, strides=strides, padding="same")(inputs)
     max_down = conv_dw_pw_block(max_down, filters // 2, activation=activation, name=name + "max_down_")
     conv_down = conv_dw_pw_block(inputs, inputs.shape[channel_axis], activation=activation, name=name + "conv_down_1_")
     conv_down = conv_dw_pw_block(conv_down, filters // 2, kernel_size=3, strides=strides, activation=activation, name=name + "conv_down_2_")
@@ -239,30 +240,29 @@ def path_aggregation_fpn(features, fpn_depth=2, use_csp_downsample=False, use_sh
 """ YOLORHead """
 
 
-def yolor_head_single(inputs, filters, num_classes=80, num_anchors=3, use_object_scores=True, activation="swish", name=""):
+def yolor_head_single(inputs, filters, num_classes=80, regression_len=4, num_anchors=3, use_object_scores=True, activation="swish", name=""):
     channel_axis = -1 if image_data_format() == "channels_last" else 1
-    initializer = initializers.truncated_normal(stddev=0.2)
 
     nn = conv_dw_pw_block(inputs, filters, 3, activation=activation, name=name + "1_")
-    nn = BiasLayer(initializer=initializer, axis=channel_axis, name=name + "shift_channel")(nn)
+    nn = BiasLayer(initializer=initializers.truncated_normal(stddev=0.2), axis=channel_axis, name=name + "shift_channel")(nn)
 
-    ouput_classes = num_classes + (5 if use_object_scores else 4)  # num_anchors = 3, num_anchors * (80 + 5) = 255
+    ouput_classes = num_classes + regression_len + (1 if use_object_scores else 0)  # num_anchors = 3, num_anchors * (80 + 5) = 255
     nn = layers.Conv2D(ouput_classes * num_anchors, kernel_size=1, name=name + "2_conv")(nn)
     nn = nn if image_data_format() == "channels_last" else layers.Permute([2, 3, 1])(nn)
     control_channels_layer = ChannelAffine(use_bias=False, axis=-1, name=name + "control_channel")
-    control_channels_layer.ww_init = initializer
+    control_channels_layer.ww_init = initializers.truncated_normal(stddev=0.2)
     nn = control_channels_layer(nn)
     # return nn
     return layers.Reshape([-1, ouput_classes], name=name + "output_reshape")(nn)
 
 
-def yolor_head(inputs, num_classes=80, num_anchors=1, use_object_scores=True, activation="swish", classifier_activation="sigmoid", name=""):
+def yolor_head(inputs, num_classes=80, regression_len=4, num_anchors=1, use_object_scores=True, activation="swish", classifier_activation="sigmoid", name=""):
     channel_axis = -1 if image_data_format() == "channels_last" else 1
     outputs = []
     for id, input in enumerate(inputs):
         cur_name = name + "{}_".format(id + 1)
         filters = int(input.shape[channel_axis] * 2)
-        out = yolor_head_single(input, filters, num_classes, num_anchors, use_object_scores, activation=activation, name=cur_name)
+        out = yolor_head_single(input, filters, num_classes, regression_len, num_anchors, use_object_scores, activation=activation, name=cur_name)
         outputs.append(out)
     # return outputs
     outputs = functional.concat(outputs, axis=1)
@@ -285,6 +285,7 @@ def YOLOR(
     use_shortcut_bn=True,
     fpn_depth=2,
     features_pick=[-3, -2, -1],
+    regression_len=4,  # bbox output len, typical value is 4, for yolov8 reg_max=16 -> regression_len = 16 * 4 == 64
     anchors_mode="yolor",
     num_anchors="auto",  # "auto" means: anchors_mode=="anchor_free" -> 1, anchors_mode=="yolor" -> 3, else 9
     use_object_scores="auto",  # "auto" means: True if anchors_mode=="anchor_free" or anchors_mode=="yolor", else False
@@ -300,8 +301,8 @@ def YOLOR(
     rescale_mode="raw01",  # For decode predictions, raw01 means input value in range [0, 1].
     kwargs=None,  # Not using, recieving parameter
 ):
-    # Regard input_shape as force using original shape if first element is None or -1,
-    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    # Regard input_shape as force using original shape if len(input_shape) == 4,
+    # else assume channel dimension is the one with min value in input_shape, and put it first or last regarding image_data_format
     input_shape = backend.align_input_shape_by_image_data_format(input_shape)
 
     if backbone is None:
@@ -325,17 +326,20 @@ def YOLOR(
     inputs = backbone.inputs[0]
 
     fpn_features = path_aggregation_fpn(features, fpn_depth, use_csp_downsample, use_shortcut_bn, activation=activation, name="pafpn_")
-    outputs = yolor_head(fpn_features, num_classes, num_anchors, use_object_scores, activation, classifier_activation, name="head_")
+    outputs = yolor_head(fpn_features, num_classes, regression_len, num_anchors, use_object_scores, activation, classifier_activation, name="head_")
     outputs = layers.Activation("linear", dtype="float32", name="outputs_fp32")(outputs)
     model = models.Model(inputs, outputs, name=model_name)
     reload_model_weights(model, PRETRAINED_DICT, "yolor", pretrained)
 
     pyramid_levels = [pyramid_levels_min, pyramid_levels_min + len(features_pick) - 1]  # -> [3, 5]
-    post_process = eval_func.DecodePredictions(backbone.input_shape[1:], pyramid_levels, anchors_mode, use_object_scores, anchor_scale)
+    post_process = eval_func.DecodePredictions(
+        backbone.input_shape[1:], pyramid_levels, anchors_mode, use_object_scores, anchor_scale, regression_len=regression_len
+    )
     add_pre_post_process(model, rescale_mode=rescale_mode, post_process=post_process)
     return model
 
 
+@register_model
 def YOLOR_CSP(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [2, 8, 8, 4]
     csp_channels = [128, 256, 512, 1024]
@@ -344,6 +348,7 @@ def YOLOR_CSP(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80,
     return YOLOR(**locals(), model_name=kwargs.pop("model_name", "yolor_csp"), **kwargs)
 
 
+@register_model
 def YOLOR_CSPX(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [3, 10, 10, 5]
     csp_channels = [160, 320, 640, 1280]
@@ -352,6 +357,7 @@ def YOLOR_CSPX(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80,
     return YOLOR(**locals(), model_name=kwargs.pop("model_name", "yolor_csp_x"), **kwargs)
 
 
+@register_model
 def YOLOR_P6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [3, 7, 7, 3, 3]
     csp_channels = [128, 256, 384, 512, 640]
@@ -363,6 +369,7 @@ def YOLOR_P6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80,
     return YOLOR(**locals(), model_name=kwargs.pop("model_name", "yolor_p6"), **kwargs)
 
 
+@register_model
 def YOLOR_W6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [3, 7, 7, 3, 3]
     csp_channels = [128, 256, 512, 768, 1024]
@@ -374,6 +381,7 @@ def YOLOR_W6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80,
     return YOLOR(**locals(), model_name=kwargs.pop("model_name", "yolor_w6"), **kwargs)
 
 
+@register_model
 def YOLOR_E6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [3, 7, 7, 3, 3]
     csp_channels = [160, 320, 640, 960, 1280]
@@ -387,6 +395,7 @@ def YOLOR_E6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80,
     return YOLOR(**locals(), model_name=kwargs.pop("model_name", "yolor_e6"), **kwargs)
 
 
+@register_model
 def YOLOR_D6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=80, backbone=None, classifier_activation="sigmoid", pretrained="coco", **kwargs):
     csp_depthes = [3, 15, 15, 7, 7]
     csp_channels = [160, 320, 640, 960, 1280]
