@@ -1,12 +1,13 @@
-import pytest
-from skimage.data import chelsea
-
 import sys
-
 sys.path.append(".")
-import keras_cv_attention_models
+import keras_cv_attention_models  # Needs to set TF_USE_LEGACY_KERAS=1 env firstly
+
+import pytest
+import numpy as np
+
 from keras_cv_attention_models.backend import models
-from keras_cv_attention_models.coco.data import COCO_80_LABEL_DICT, COCO_90_LABEL_DICT
+from keras_cv_attention_models.coco.info import COCO_80_LABEL_DICT, COCO_90_LABEL_DICT
+from keras_cv_attention_models.test_images import cat
 
 """ Recognition models defination """
 
@@ -184,7 +185,7 @@ def test_WaveMLP_defination():
 
 def test_Beit_new_shape_predict():
     mm = keras_cv_attention_models.beit.BeitBasePatch16(input_shape=(320, 320, 3))
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -192,7 +193,7 @@ def test_Beit_new_shape_predict():
 
 def test_BotNet_new_shape_predict():
     mm = keras_cv_attention_models.botnet.BotNextECA26T(input_shape=(512, 512, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -200,7 +201,7 @@ def test_BotNet_new_shape_predict():
 
 def test_CAFormerS18_new_shape_predict():
     mm = keras_cv_attention_models.caformer.CAFormerS18(input_shape=(112, 193, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -208,7 +209,7 @@ def test_CAFormerS18_new_shape_predict():
 
 def test_CMTTiny_new_shape_predict():
     mm = keras_cv_attention_models.cmt.CMTTiny(input_shape=(117, 192, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -216,7 +217,7 @@ def test_CMTTiny_new_shape_predict():
 
 def test_CoaT_new_shape_predict():
     mm = keras_cv_attention_models.coat.CoaTLiteMini(input_shape=(193, 117, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -224,7 +225,15 @@ def test_CoaT_new_shape_predict():
 
 def test_CoAtNet_new_shape_predict():
     mm = keras_cv_attention_models.coatnet.CoAtNet0(input_shape=(320, 320, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_CotNet_new_shape_predict():
+    mm = keras_cv_attention_models.cotnet.CotNet50(input_shape=(193, 117, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -232,7 +241,7 @@ def test_CoAtNet_new_shape_predict():
 
 def test_ConvNeXt_predict():
     mm = keras_cv_attention_models.convnext.ConvNeXtTiny(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -240,7 +249,15 @@ def test_ConvNeXt_predict():
 
 def test_ConvNeXtV2_predict():
     mm = keras_cv_attention_models.convnext.ConvNeXtV2Atto(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_CSPNeXtTiny_dynamic_predict():
+    mm = keras_cv_attention_models.cspnext.CSPNeXtTiny(input_shape=(None, None, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -248,7 +265,23 @@ def test_ConvNeXtV2_predict():
 
 def test_DaViT_T_new_shape_predict():
     mm = keras_cv_attention_models.davit.DaViT_T(input_shape=(376, 227, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_DINOv2_ViT_Small14_new_shape_predict():
+    mm = keras_cv_attention_models.dinov2.DINOv2_ViT_Small14(input_shape=(224, 224, 3), patch_size=32)
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_EVA02TinyPatch14_new_shape_predict():
+    mm = keras_cv_attention_models.eva02.EVA02TinyPatch14(input_shape=(224, 224, 3), patch_size=16)
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -256,15 +289,15 @@ def test_DaViT_T_new_shape_predict():
 
 def test_EdgeNeXt_XX_Small_new_shape_predict():
     mm = keras_cv_attention_models.edgenext.EdgeNeXt_XX_Small(input_shape=(174, 269, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
 
 
 def test_EfficientFormerL1_new_shape_predict():
-    mm = keras_cv_attention_models.efficientformer.EfficientFormerL1(input_shape=(192, 113, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    mm = keras_cv_attention_models.efficientformer.EfficientFormerL1(input_shape=(192, 113, 3), use_distillation=True, pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     pred = (pred[0] + pred[1]) / 2
     out = mm.decode_predictions(pred)[0][0]
 
@@ -272,8 +305,8 @@ def test_EfficientFormerL1_new_shape_predict():
 
 
 def test_EfficientFormerV2S0_new_shape_predict():
-    mm = keras_cv_attention_models.efficientformer.EfficientFormerV2S0(input_shape=(192, 113, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    mm = keras_cv_attention_models.efficientformer.EfficientFormerV2S0(input_shape=(192, 113, 3), use_distillation=True, pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     pred = (pred[0] + pred[1]) / 2
     out = mm.decode_predictions(pred)[0][0]
 
@@ -282,7 +315,7 @@ def test_EfficientFormerV2S0_new_shape_predict():
 
 def test_EfficientNetV1B1_noisy_student_predict():
     mm = keras_cv_attention_models.efficientnet.EfficientNetV1B1(pretrained="noisy_student")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -290,7 +323,23 @@ def test_EfficientNetV1B1_noisy_student_predict():
 
 def test_EfficientNetV2B0_predict():
     mm = keras_cv_attention_models.efficientnet.EfficientNetV2B0(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_EfficientViT_B1_new_shape_predict():
+    mm = keras_cv_attention_models.efficientvit.EfficientViT_B1(input_shape=(192, 127, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_EfficientViT_M0_new_shape_predict():
+    mm = keras_cv_attention_models.efficientvit.EfficientViT_M0(input_shape=(192, 127, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -298,7 +347,31 @@ def test_EfficientNetV2B0_predict():
 
 def test_FasterNetT2_dynamic_predict():
     mm = keras_cv_attention_models.fasternet.FasterNetT2(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_FasterViT0_new_shape_predict():
+    mm = keras_cv_attention_models.fastervit.FasterViT0(input_shape=(192, 127, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_FastViT_T8_dynamic_predict():
+    mm = keras_cv_attention_models.fastvit.FastViT_T8(input_shape=(None, None, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(192, 127, 3)))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_FastViT_SA12_new_shape_predict():
+    mm = keras_cv_attention_models.fastvit.FastViT_SA12(input_shape=(192, 127, 3), pretrained="distill")
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -306,7 +379,7 @@ def test_FasterNetT2_dynamic_predict():
 
 def test_FBNetV3B_dynamic_predict():
     mm = keras_cv_attention_models.fbnetv3.FBNetV3B(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -314,7 +387,7 @@ def test_FBNetV3B_dynamic_predict():
 
 def test_GCViT_XXTiny_new_shape_predict():
     mm = keras_cv_attention_models.gcvit.GCViT_XXTiny(input_shape=(128, 192, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -322,7 +395,7 @@ def test_GCViT_XXTiny_new_shape_predict():
 
 def test_GhostNetV2_100_dynamic_predict():
     mm = keras_cv_attention_models.ghostnet.GhostNetV2_100(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(112, 157, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(112, 157, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -330,7 +403,31 @@ def test_GhostNetV2_100_dynamic_predict():
 
 def test_GPViT_L1_new_shape_predict():
     mm = keras_cv_attention_models.gpvit.GPViT_L1(input_shape=(112, 157, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_HaloRegNetZB_new_shape_predict():
+    mm = keras_cv_attention_models.halonet.HaloRegNetZB(input_shape=(193, 117, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_HieraTiny_predict():
+    mm = keras_cv_attention_models.hiera.HieraTiny()
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_HorNetTiny_dynamic_predict():
+    mm = keras_cv_attention_models.hornet.HorNetTiny(input_shape=(None, None, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -338,7 +435,15 @@ def test_GPViT_L1_new_shape_predict():
 
 def test_IFormerSmall_new_shape_predict():
     mm = keras_cv_attention_models.iformer.IFormerSmall(input_shape=(174, 255, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_InceptionNeXtTiny_dynamic_predict():
+    mm = keras_cv_attention_models.inceptionnext.InceptionNeXtTiny(input_shape=(None, None, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -346,15 +451,15 @@ def test_IFormerSmall_new_shape_predict():
 
 def test_LCNet050_dynamic_predict():
     mm = keras_cv_attention_models.lcnet.LCNet050(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
 
 
 def test_LeViT128S_predict():
-    mm = keras_cv_attention_models.levit.LeViT128S(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    mm = keras_cv_attention_models.levit.LeViT128S(use_distillation=True, pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     pred = (pred[0] + pred[1]) / 2
     out = mm.decode_predictions(pred)[0][0]
 
@@ -362,8 +467,8 @@ def test_LeViT128S_predict():
 
 
 def test_LeViT128S_new_shape_predict():
-    mm = keras_cv_attention_models.levit.LeViT128S(input_shape=(292, 213, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    mm = keras_cv_attention_models.levit.LeViT128S(use_distillation=True, input_shape=(292, 213, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     pred = (pred[0] + pred[1]) / 2
     out = mm.decode_predictions(pred)[0][0]
 
@@ -372,7 +477,7 @@ def test_LeViT128S_new_shape_predict():
 
 def test_MaxViT_Tiny_new_shape_predict():
     mm = keras_cv_attention_models.maxvit.MaxViT_Tiny(input_shape=(174, 255, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -380,7 +485,7 @@ def test_MaxViT_Tiny_new_shape_predict():
 
 def test_MobileNetV3Small075_dynamic_predict():
     mm = keras_cv_attention_models.mobilenetv3.MobileNetV3Small075(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -388,7 +493,7 @@ def test_MobileNetV3Small075_dynamic_predict():
 
 def test_MobileViT_XXS_predict():
     mm = keras_cv_attention_models.mobilevit.MobileViT_XXS(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -396,7 +501,7 @@ def test_MobileViT_XXS_predict():
 
 def test_MobileViT_V2_050_predict():
     mm = keras_cv_attention_models.mobilevit.MobileViT_V2_050(input_shape=(252, 224, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -404,7 +509,23 @@ def test_MobileViT_V2_050_predict():
 
 def test_MogaNetXtiny_dynamic_predict():
     mm = keras_cv_attention_models.moganet.MogaNetXtiny(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_NAT_Mini_new_shape_predict():
+    mm = keras_cv_attention_models.nat.NAT_Mini(input_shape=(174, 255, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_DiNAT_Mini_new_shape_predict():
+    mm = keras_cv_attention_models.nat.DiNAT_Mini(input_shape=(174, 255, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -412,7 +533,7 @@ def test_MogaNetXtiny_dynamic_predict():
 
 def test_PVT_V2B0_new_shape_predict():
     mm = keras_cv_attention_models.pvt.PVT_V2B0(input_shape=(174, 255, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -420,7 +541,7 @@ def test_PVT_V2B0_new_shape_predict():
 
 def test_PVT_V2B2_linear_new_shape_predict():
     mm = keras_cv_attention_models.pvt.PVT_V2B2_linear(input_shape=(193, 255, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -428,7 +549,15 @@ def test_PVT_V2B2_linear_new_shape_predict():
 
 def test_RegNetZB16_predict():
     mm = keras_cv_attention_models.resnet_family.RegNetZB16(pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_RepViT_M1_dynamic_predict():
+    mm = keras_cv_attention_models.repvit.RepViT_M09(input_shape=(None, None, 3), use_distillation=False, pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 192, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -436,7 +565,7 @@ def test_RegNetZB16_predict():
 
 def test_ResNest50_dynamic_predict():
     mm = keras_cv_attention_models.resnest.ResNest50(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 192, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 192, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -444,7 +573,7 @@ def test_ResNest50_dynamic_predict():
 
 def test_TinyViT_5M_new_shape_predict():
     mm = keras_cv_attention_models.tinyvit.TinyViT_5M(input_shape=(160, 160, 3))
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -452,7 +581,7 @@ def test_TinyViT_5M_new_shape_predict():
 
 def test_TinyNetD_dynamic_predict():
     mm = keras_cv_attention_models.tinynet.TinyNetD(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=(160, 256, 3)))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -460,7 +589,7 @@ def test_TinyNetD_dynamic_predict():
 
 def test_SwinTransformerV2Tiny_window8_new_shape_predict():
     mm = keras_cv_attention_models.swin_transformer_v2.SwinTransformerV2Tiny_window8(input_shape=(160, 160, 3))
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -468,7 +597,7 @@ def test_SwinTransformerV2Tiny_window8_new_shape_predict():
 
 def test_UniformerSmall64_new_shape_predict():
     mm = keras_cv_attention_models.uniformer.UniformerSmall64(input_shape=(512, 512, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -476,7 +605,15 @@ def test_UniformerSmall64_new_shape_predict():
 
 def test_WaveMLP_T_dynamic_predict():
     mm = keras_cv_attention_models.wave_mlp.WaveMLP_T(input_shape=(None, None, 3), pretrained="imagenet")
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=[320, 320, 3]))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=[320, 320, 3]))
+    out = mm.decode_predictions(pred)[0][0]
+
+    assert out[1] == "Egyptian_cat"
+
+
+def test_VanillaNet5_dynamic_predict():
+    mm = keras_cv_attention_models.vanillanet.VanillaNet5(input_shape=(None, None, 3), pretrained="imagenet")
+    pred = mm(mm.preprocess_input(cat(), input_shape=(160, 256, 3)))
     out = mm.decode_predictions(pred)[0][0]
 
     assert out[1] == "Egyptian_cat"
@@ -487,7 +624,7 @@ def test_WaveMLP_T_dynamic_predict():
 
 def test_EfficientDetD0_predict():
     mm = keras_cv_attention_models.efficientdet.EfficientDetD0(pretrained="coco")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     assert pred.shape == (1, 49104, 94)
 
     pred_label = mm.decode_predictions(pred)[0][1].numpy()
@@ -504,7 +641,7 @@ def test_EfficientDet_header():
 def test_EfficientDetD1_dynamic_predict():
     mm = keras_cv_attention_models.efficientdet.EfficientDetD1(input_shape=(None, None, 3), pretrained="coco")
     input_shape = (376, 227, 3)
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=input_shape))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=input_shape))
     assert pred.shape == (1, 16641, 94)
 
     pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
@@ -514,7 +651,7 @@ def test_EfficientDetD1_dynamic_predict():
 def test_EfficientDetLite1_dynamic_predict():
     mm = keras_cv_attention_models.efficientdet.EfficientDetLite1(input_shape=(None, None, 3), pretrained="coco")
     input_shape = (376, 227, 3)
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=input_shape))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=input_shape))
     assert pred.shape == (1, 16641, 94)
 
     pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
@@ -523,7 +660,7 @@ def test_EfficientDetLite1_dynamic_predict():
 
 def test_YOLOR_CSP_predict():
     mm = keras_cv_attention_models.yolor.YOLOR_CSP(pretrained="coco")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     assert pred.shape == (1, 25200, 85)
 
     pred_label = mm.decode_predictions(pred)[0][1].numpy()
@@ -540,7 +677,7 @@ def test_YOLOR_header():
 def test_YOLOR_CSP_dynamic_predict():
     mm = keras_cv_attention_models.yolor.YOLOR_CSP(input_shape=(None, None, 3), pretrained="coco")
     input_shape = (188, 275, 3)
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=input_shape))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=input_shape))
     assert pred.shape == (1, 3330, 85)
 
     pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
@@ -549,7 +686,7 @@ def test_YOLOR_CSP_dynamic_predict():
 
 def test_YOLOXTiny_predict():
     mm = keras_cv_attention_models.yolox.YOLOXTiny(pretrained="coco")
-    pred = mm(mm.preprocess_input(chelsea()[:, :, ::-1]))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()[:, :, ::-1]))
     assert pred.shape == (1, 3549, 85)
 
     pred_label = mm.decode_predictions(pred)[0][1].numpy()
@@ -566,7 +703,7 @@ def test_YOLOX_header():
 def test_YOLOXS_dynamic_predict():
     mm = keras_cv_attention_models.yolox.YOLOXS(input_shape=(None, None, 3), pretrained="coco")
     input_shape = (188, 276, 3)
-    pred = mm(mm.preprocess_input(chelsea()[:, :, ::-1], input_shape=input_shape))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()[:, :, ::-1], input_shape=input_shape))
     assert pred.shape == (1, 1110, 85)
 
     pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
@@ -575,7 +712,7 @@ def test_YOLOXS_dynamic_predict():
 
 def test_YOLOV7_Tiny_predict():
     mm = keras_cv_attention_models.yolov7.YOLOV7_Tiny(pretrained="coco")
-    pred = mm(mm.preprocess_input(chelsea()))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat()))
     assert pred.shape == (1, 10647, 85)
 
     pred_label = mm.decode_predictions(pred)[0][1].numpy()
@@ -585,8 +722,56 @@ def test_YOLOV7_Tiny_predict():
 def test_YOLOV7_Tiny_dynamic_predict():
     mm = keras_cv_attention_models.yolov7.YOLOV7_Tiny(input_shape=(None, None, 3), pretrained="coco")
     input_shape = (188, 276, 3)
-    pred = mm(mm.preprocess_input(chelsea(), input_shape=input_shape))  # Chelsea the cat
+    pred = mm(mm.preprocess_input(cat(), input_shape=input_shape))
     assert pred.shape == (1, 3330, 85)
 
     pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
     assert COCO_80_LABEL_DICT[pred_label[0]] == "cat"
+
+
+def test_YOLOV8_S_predict():
+    mm = keras_cv_attention_models.yolov8.YOLOV8_S(pretrained="coco")
+    pred = mm(mm.preprocess_input(cat()))
+    assert pred.shape == (1, 8400, 144)
+
+    pred_label = mm.decode_predictions(pred)[0][1].numpy()
+    assert COCO_80_LABEL_DICT[pred_label[0]] == "cat"
+
+
+def test_YOLOV8_S_dynamic_predict():
+    mm = keras_cv_attention_models.yolov8.YOLOV8_S(input_shape=(None, None, 3), pretrained="coco")
+    input_shape = (188, 276, 3)
+    pred = mm(mm.preprocess_input(cat(), input_shape=input_shape))
+    assert pred.shape == (1, 1110, 144)
+
+    pred_label = mm.decode_predictions(pred, input_shape=input_shape)[0][1].numpy()
+    assert COCO_80_LABEL_DICT[pred_label[0]] == "cat"
+
+
+""" Stable Diffusion """
+
+
+def test_stable_diffusion_no_weights_predict():
+    mm = keras_cv_attention_models.stable_diffusion.StableDiffusion(pretrained=None)
+    image = keras_cv_attention_models.backend.numpy_image_resize(cat(), [256, 256])
+    out = mm("hello world", image=image, init_step=49, strength=0.02, inpaint_mask=[0.5, 0, 1, 1])  # Run only 1 step
+    if keras_cv_attention_models.backend.is_torch_backend:
+        out = out.detach().cpu().numpy()
+        assert out.shape == (1, 3, 256, 256)
+    else:
+        out = out.numpy()
+        assert out.shape == (1, 256, 256, 3)
+    assert out.min() > -8 and out.max() < 8  # It should be within this range
+
+
+""" Segment Anything """
+
+
+def test_MobileSAM_predict():
+    mm = keras_cv_attention_models.segment_anything.MobileSAM()
+    points, labels = np.array([(0.5, 0.8)]), np.array([1])
+    masks, iou_predictions, low_res_masks = mm(cat(), points, labels)
+
+    assert masks.shape == (4, 512, 512) and iou_predictions.shape == (4,) and low_res_masks.shape == (4, 256, 256)
+    assert np.allclose(iou_predictions, np.array([0.98725945, 0.83492416, 0.9997821, 0.96904826]), atol=1e-3)
+    assert np.allclose([ii.sum() for ii in masks], [140151, 121550, 139295, 149360], atol=10)
